# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.common.types import criteria
from google.ads.googleads.v22.enums.types import ad_strength as gage_ad_strength
from google.ads.googleads.v22.enums.types import (
    app_bidding_goal as gage_app_bidding_goal,
)
from google.ads.googleads.v22.enums.types import keyword_match_type
from google.ads.googleads.v22.enums.types import recommendation_type
from google.ads.googleads.v22.enums.types import (
    shopping_add_products_to_campaign_recommendation_enum,
)
from google.ads.googleads.v22.enums.types import (
    target_cpa_opt_in_recommendation_goal,
)
from google.ads.googleads.v22.resources.types import ad as gagr_ad
from google.ads.googleads.v22.resources.types import asset


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "Recommendation",
    },
)


class Recommendation(proto.Message):
    r"""A recommendation.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the recommendation.

            ``customers/{customer_id}/recommendations/{recommendation_id}``
        type_ (google.ads.googleads.v22.enums.types.RecommendationTypeEnum.RecommendationType):
            Output only. The type of recommendation.
        impact (google.ads.googleads.v22.resources.types.Recommendation.RecommendationImpact):
            Output only. The impact on account
            performance as a result of applying the
            recommendation.
        campaign_budget (str):
            Output only. The budget targeted by this recommendation.
            This will be set only when the recommendation affects a
            single campaign budget.

            This field will be set for the following recommendation
            types: CAMPAIGN_BUDGET, FORECASTING_CAMPAIGN_BUDGET,
            MARGINAL_ROI_CAMPAIGN_BUDGET, MOVE_UNUSED_BUDGET

            This field is a member of `oneof`_ ``_campaign_budget``.
        campaign (str):
            Output only. The campaign targeted by this recommendation.

            This field will be set for the following recommendation
            types: CALL_EXTENSION, CALLOUT_EXTENSION,
            ENHANCED_CPC_OPT_IN, USE_BROAD_MATCH_KEYWORD, KEYWORD,
            KEYWORD_MATCH_TYPE,
            UPGRADE_LOCAL_CAMPAIGN_TO_PERFORMANCE_MAX,
            MAXIMIZE_CLICKS_OPT_IN, MAXIMIZE_CONVERSIONS_OPT_IN,
            OPTIMIZE_AD_ROTATION, RESPONSIVE_SEARCH_AD,
            RESPONSIVE_SEARCH_AD_ASSET, SEARCH_PARTNERS_OPT_IN,
            DISPLAY_EXPANSION_OPT_IN, SITELINK_EXTENSION,
            TARGET_CPA_OPT_IN, TARGET_ROAS_OPT_IN, TEXT_AD,
            UPGRADE_SMART_SHOPPING_CAMPAIGN_TO_PERFORMANCE_MAX,
            RAISE_TARGET_CPA_BID_TOO_LOW, FORECASTING_SET_TARGET_ROAS,
            SHOPPING_ADD_AGE_GROUP, SHOPPING_ADD_COLOR,
            SHOPPING_ADD_GENDER, SHOPPING_ADD_SIZE, SHOPPING_ADD_GTIN,
            SHOPPING_ADD_MORE_IDENTIFIERS,
            SHOPPING_ADD_PRODUCTS_TO_CAMPAIGN,
            SHOPPING_FIX_DISAPPROVED_PRODUCTS,
            SHOPPING_MIGRATE_REGULAR_SHOPPING_CAMPAIGN_OFFERS_TO_PERFORMANCE_MAX,
            DYNAMIC_IMAGE_EXTENSION_OPT_IN, RAISE_TARGET_CPA,
            LOWER_TARGET_ROAS, FORECASTING_SET_TARGET_CPA,
            SET_TARGET_CPA, SET_TARGET_ROAS,
            MAXIMIZE_CONVERSION_VALUE_OPT_IN,
            IMPROVE_GOOGLE_TAG_COVERAGE,
            PERFORMANCE_MAX_FINAL_URL_OPT_IN

            This field is a member of `oneof`_ ``_campaign``.
        ad_group (str):
            Output only. The ad group targeted by this recommendation.
            This will be set only when the recommendation affects a
            single ad group.

            This field will be set for the following recommendation
            types: KEYWORD, OPTIMIZE_AD_ROTATION, RESPONSIVE_SEARCH_AD,
            RESPONSIVE_SEARCH_AD_ASSET, TEXT_AD

            This field is a member of `oneof`_ ``_ad_group``.
        dismissed (bool):
            Output only. Whether the recommendation is
            dismissed or not.

            This field is a member of `oneof`_ ``_dismissed``.
        campaigns (MutableSequence[str]):
            Output only. The campaigns targeted by this recommendation.

            This field will be set for the following recommendation
            types: CAMPAIGN_BUDGET, FORECASTING_CAMPAIGN_BUDGET,
            MARGINAL_ROI_CAMPAIGN_BUDGET and MOVE_UNUSED_BUDGET
        campaign_budget_recommendation (google.ads.googleads.v22.resources.types.Recommendation.CampaignBudgetRecommendation):
            Output only. The campaign budget
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        forecasting_campaign_budget_recommendation (google.ads.googleads.v22.resources.types.Recommendation.CampaignBudgetRecommendation):
            Output only. The forecasting campaign budget
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        keyword_recommendation (google.ads.googleads.v22.resources.types.Recommendation.KeywordRecommendation):
            Output only. The keyword recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        text_ad_recommendation (google.ads.googleads.v22.resources.types.Recommendation.TextAdRecommendation):
            Output only. Add expanded text ad
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        target_cpa_opt_in_recommendation (google.ads.googleads.v22.resources.types.Recommendation.TargetCpaOptInRecommendation):
            Output only. The TargetCPA opt-in
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        maximize_conversions_opt_in_recommendation (google.ads.googleads.v22.resources.types.Recommendation.MaximizeConversionsOptInRecommendation):
            Output only. The MaximizeConversions Opt-In
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        enhanced_cpc_opt_in_recommendation (google.ads.googleads.v22.resources.types.Recommendation.EnhancedCpcOptInRecommendation):
            Output only. The Enhanced Cost-Per-Click
            Opt-In recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        search_partners_opt_in_recommendation (google.ads.googleads.v22.resources.types.Recommendation.SearchPartnersOptInRecommendation):
            Output only. The Search Partners Opt-In
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        maximize_clicks_opt_in_recommendation (google.ads.googleads.v22.resources.types.Recommendation.MaximizeClicksOptInRecommendation):
            Output only. The MaximizeClicks Opt-In
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        optimize_ad_rotation_recommendation (google.ads.googleads.v22.resources.types.Recommendation.OptimizeAdRotationRecommendation):
            Output only. The Optimize Ad Rotation
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        keyword_match_type_recommendation (google.ads.googleads.v22.resources.types.Recommendation.KeywordMatchTypeRecommendation):
            Output only. The keyword match type
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        move_unused_budget_recommendation (google.ads.googleads.v22.resources.types.Recommendation.MoveUnusedBudgetRecommendation):
            Output only. The move unused budget
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        target_roas_opt_in_recommendation (google.ads.googleads.v22.resources.types.Recommendation.TargetRoasOptInRecommendation):
            Output only. The Target ROAS opt-in
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        responsive_search_ad_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ResponsiveSearchAdRecommendation):
            Output only. The add responsive search ad
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        marginal_roi_campaign_budget_recommendation (google.ads.googleads.v22.resources.types.Recommendation.CampaignBudgetRecommendation):
            Output only. The marginal ROI campaign budget
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        use_broad_match_keyword_recommendation (google.ads.googleads.v22.resources.types.Recommendation.UseBroadMatchKeywordRecommendation):
            Output only. The use broad match keyword
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        responsive_search_ad_asset_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ResponsiveSearchAdAssetRecommendation):
            Output only. The responsive search ad asset
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        upgrade_smart_shopping_campaign_to_performance_max_recommendation (google.ads.googleads.v22.resources.types.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation):
            Output only. The upgrade a Smart Shopping
            campaign to a Performance Max campaign
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        responsive_search_ad_improve_ad_strength_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation):
            Output only. The responsive search ad improve
            ad strength recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        display_expansion_opt_in_recommendation (google.ads.googleads.v22.resources.types.Recommendation.DisplayExpansionOptInRecommendation):
            Output only. The Display Expansion opt-in
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        upgrade_local_campaign_to_performance_max_recommendation (google.ads.googleads.v22.resources.types.Recommendation.UpgradeLocalCampaignToPerformanceMaxRecommendation):
            Output only. The upgrade a Local campaign to
            a Performance Max campaign recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        raise_target_cpa_bid_too_low_recommendation (google.ads.googleads.v22.resources.types.Recommendation.RaiseTargetCpaBidTooLowRecommendation):
            Output only. The raise target CPA bid too low
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        forecasting_set_target_roas_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ForecastingSetTargetRoasRecommendation):
            Output only. The forecasting set target ROAS
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        callout_asset_recommendation (google.ads.googleads.v22.resources.types.Recommendation.CalloutAssetRecommendation):
            Output only. The callout asset
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        sitelink_asset_recommendation (google.ads.googleads.v22.resources.types.Recommendation.SitelinkAssetRecommendation):
            Output only. The sitelink asset
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        call_asset_recommendation (google.ads.googleads.v22.resources.types.Recommendation.CallAssetRecommendation):
            Output only. The call asset recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        shopping_add_age_group_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ShoppingOfferAttributeRecommendation):
            Output only. The shopping add age group
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        shopping_add_color_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ShoppingOfferAttributeRecommendation):
            Output only. The shopping add color
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        shopping_add_gender_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ShoppingOfferAttributeRecommendation):
            Output only. The shopping add gender
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        shopping_add_gtin_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ShoppingOfferAttributeRecommendation):
            Output only. The shopping add GTIN
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        shopping_add_more_identifiers_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ShoppingOfferAttributeRecommendation):
            Output only. The shopping add more
            identifiers recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        shopping_add_size_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ShoppingOfferAttributeRecommendation):
            Output only. The shopping add size
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        shopping_add_products_to_campaign_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ShoppingAddProductsToCampaignRecommendation):
            Output only. The shopping add products to
            campaign recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        shopping_fix_disapproved_products_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ShoppingFixDisapprovedProductsRecommendation):
            Output only. The shopping fix disapproved
            products recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        shopping_target_all_offers_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ShoppingTargetAllOffersRecommendation):
            Output only. The shopping target all offers
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        shopping_fix_suspended_merchant_center_account_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ShoppingMerchantCenterAccountSuspensionRecommendation):
            Output only. The shopping fix suspended
            Merchant Center account recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        shopping_fix_merchant_center_account_suspension_warning_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ShoppingMerchantCenterAccountSuspensionRecommendation):
            Output only. The shopping fix Merchant Center
            account suspension warning recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        shopping_migrate_regular_shopping_campaign_offers_to_performance_max_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ShoppingMigrateRegularShoppingCampaignOffersToPerformanceMaxRecommendation):
            Output only. The shopping migrate Regular
            Shopping Campaign offers to Performance Max
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        dynamic_image_extension_opt_in_recommendation (google.ads.googleads.v22.resources.types.Recommendation.DynamicImageExtensionOptInRecommendation):
            Output only. Recommendation to enable dynamic
            image extensions on the account, allowing Google
            to find the best images from ad landing pages
            and complement text ads.

            This field is a member of `oneof`_ ``recommendation``.
        raise_target_cpa_recommendation (google.ads.googleads.v22.resources.types.Recommendation.RaiseTargetCpaRecommendation):
            Output only. Recommendation to raise Target
            CPA.

            This field is a member of `oneof`_ ``recommendation``.
        lower_target_roas_recommendation (google.ads.googleads.v22.resources.types.Recommendation.LowerTargetRoasRecommendation):
            Output only. Recommendation to lower Target
            ROAS.

            This field is a member of `oneof`_ ``recommendation``.
        performance_max_opt_in_recommendation (google.ads.googleads.v22.resources.types.Recommendation.PerformanceMaxOptInRecommendation):
            Output only. The Performance Max Opt In
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        improve_performance_max_ad_strength_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ImprovePerformanceMaxAdStrengthRecommendation):
            Output only. The improve Performance Max ad
            strength recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        migrate_dynamic_search_ads_campaign_to_performance_max_recommendation (google.ads.googleads.v22.resources.types.Recommendation.MigrateDynamicSearchAdsCampaignToPerformanceMaxRecommendation):
            Output only. The Dynamic Search Ads to
            Performance Max migration recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        forecasting_set_target_cpa_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ForecastingSetTargetCpaRecommendation):
            Output only. The forecasting set target CPA
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        set_target_cpa_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ForecastingSetTargetCpaRecommendation):
            Output only. The set target CPA
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        set_target_roas_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ForecastingSetTargetRoasRecommendation):
            Output only. The set target ROAS
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        maximize_conversion_value_opt_in_recommendation (google.ads.googleads.v22.resources.types.Recommendation.MaximizeConversionValueOptInRecommendation):
            Output only. The Maximize Conversion Value
            opt-in recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        improve_google_tag_coverage_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ImproveGoogleTagCoverageRecommendation):
            Output only. Recommendation to deploy Google
            Tag on more pages.

            This field is a member of `oneof`_ ``recommendation``.
        performance_max_final_url_opt_in_recommendation (google.ads.googleads.v22.resources.types.Recommendation.PerformanceMaxFinalUrlOptInRecommendation):
            Output only. Recommendation to turn on Final
            URL expansion for your Performance Max
            campaigns.

            This field is a member of `oneof`_ ``recommendation``.
        refresh_customer_match_list_recommendation (google.ads.googleads.v22.resources.types.Recommendation.RefreshCustomerMatchListRecommendation):
            Output only. The refresh customer list
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        custom_audience_opt_in_recommendation (google.ads.googleads.v22.resources.types.Recommendation.CustomAudienceOptInRecommendation):
            Output only. The custom audience opt in
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        lead_form_asset_recommendation (google.ads.googleads.v22.resources.types.Recommendation.LeadFormAssetRecommendation):
            Output only. The lead form asset
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        improve_demand_gen_ad_strength_recommendation (google.ads.googleads.v22.resources.types.Recommendation.ImproveDemandGenAdStrengthRecommendation):
            Output only. The improve Demand Gen ad
            strength recommendation.

            This field is a member of `oneof`_ ``recommendation``.
    """

    class MerchantInfo(proto.Message):
        r"""The Merchant Center account details.

        Attributes:
            id (int):
                Output only. The Merchant Center account ID.
            name (str):
                Output only. The name of the Merchant Center
                account.
            multi_client (bool):
                Output only. Whether the Merchant Center
                account is a Multi-Client account (MCA).
        """

        id: int = proto.Field(
            proto.INT64,
            number=1,
        )
        name: str = proto.Field(
            proto.STRING,
            number=2,
        )
        multi_client: bool = proto.Field(
            proto.BOOL,
            number=3,
        )

    class RecommendationImpact(proto.Message):
        r"""The impact of making the change as described in the
        recommendation. Some types of recommendations may not have
        impact information.

        Attributes:
            base_metrics (google.ads.googleads.v22.resources.types.Recommendation.RecommendationMetrics):
                Output only. Base metrics at the time the
                recommendation was generated.
            potential_metrics (google.ads.googleads.v22.resources.types.Recommendation.RecommendationMetrics):
                Output only. Estimated metrics if the
                recommendation is applied.
        """

        base_metrics: "Recommendation.RecommendationMetrics" = proto.Field(
            proto.MESSAGE,
            number=1,
            message="Recommendation.RecommendationMetrics",
        )
        potential_metrics: "Recommendation.RecommendationMetrics" = proto.Field(
            proto.MESSAGE,
            number=2,
            message="Recommendation.RecommendationMetrics",
        )

    class RecommendationMetrics(proto.Message):
        r"""Weekly account performance metrics. For some recommendation
        types, these are averaged over the past 90-day period and hence
        can be fractional.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            impressions (float):
                Output only. Number of ad impressions.

                This field is a member of `oneof`_ ``_impressions``.
            clicks (float):
                Output only. Number of ad clicks.

                This field is a member of `oneof`_ ``_clicks``.
            cost_micros (int):
                Output only. Cost (in micros) for
                advertising, in the local currency for the
                account.

                This field is a member of `oneof`_ ``_cost_micros``.
            conversions (float):
                Output only. Number of conversions.

                This field is a member of `oneof`_ ``_conversions``.
            conversions_value (float):
                Output only. Sum of the conversion value of
                the conversions.

                This field is a member of `oneof`_ ``_conversions_value``.
            video_views (float):
                Output only. Number of video views for a
                video ad campaign.

                This field is a member of `oneof`_ ``_video_views``.
        """

        impressions: float = proto.Field(
            proto.DOUBLE,
            number=6,
            optional=True,
        )
        clicks: float = proto.Field(
            proto.DOUBLE,
            number=7,
            optional=True,
        )
        cost_micros: int = proto.Field(
            proto.INT64,
            number=8,
            optional=True,
        )
        conversions: float = proto.Field(
            proto.DOUBLE,
            number=9,
            optional=True,
        )
        conversions_value: float = proto.Field(
            proto.DOUBLE,
            number=11,
            optional=True,
        )
        video_views: float = proto.Field(
            proto.DOUBLE,
            number=10,
            optional=True,
        )

    class CampaignBudgetRecommendation(proto.Message):
        r"""The budget recommendation for budget constrained campaigns.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            current_budget_amount_micros (int):
                Output only. The current budget amount in
                micros.

                This field is a member of `oneof`_ ``_current_budget_amount_micros``.
            recommended_budget_amount_micros (int):
                Output only. The recommended budget amount in
                micros.

                This field is a member of `oneof`_ ``_recommended_budget_amount_micros``.
            budget_options (MutableSequence[google.ads.googleads.v22.resources.types.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption]):
                Output only. The budget amounts and
                associated impact estimates for some values of
                possible budget amounts.
        """

        class CampaignBudgetRecommendationOption(proto.Message):
            r"""The impact estimates for a given budget amount.

            .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

            Attributes:
                budget_amount_micros (int):
                    Output only. The budget amount for this
                    option.

                    This field is a member of `oneof`_ ``_budget_amount_micros``.
                impact (google.ads.googleads.v22.resources.types.Recommendation.RecommendationImpact):
                    Output only. The impact estimate if budget is
                    changed to amount specified in this option.
            """

            budget_amount_micros: int = proto.Field(
                proto.INT64,
                number=3,
                optional=True,
            )
            impact: "Recommendation.RecommendationImpact" = proto.Field(
                proto.MESSAGE,
                number=2,
                message="Recommendation.RecommendationImpact",
            )

        current_budget_amount_micros: int = proto.Field(
            proto.INT64,
            number=7,
            optional=True,
        )
        recommended_budget_amount_micros: int = proto.Field(
            proto.INT64,
            number=8,
            optional=True,
        )
        budget_options: MutableSequence[
            "Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption"
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=3,
            message="Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption",
        )

    class KeywordRecommendation(proto.Message):
        r"""The keyword recommendation.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            keyword (google.ads.googleads.v22.common.types.KeywordInfo):
                Output only. The recommended keyword.
            search_terms (MutableSequence[google.ads.googleads.v22.resources.types.Recommendation.KeywordRecommendation.SearchTerm]):
                Output only. A list of search terms this
                keyword matches. The same search term may be
                repeated for multiple keywords.
            recommended_cpc_bid_micros (int):
                Output only. The recommended CPC
                (cost-per-click) bid.

                This field is a member of `oneof`_ ``_recommended_cpc_bid_micros``.
        """

        class SearchTerm(proto.Message):
            r"""Information about a search term as related to a keyword
            recommendation.

            Attributes:
                text (str):
                    Output only. The text of the search term.
                estimated_weekly_search_count (int):
                    Output only. Estimated number of historical
                    weekly searches for this search term.
            """

            text: str = proto.Field(
                proto.STRING,
                number=1,
            )
            estimated_weekly_search_count: int = proto.Field(
                proto.INT64,
                number=2,
            )

        keyword: criteria.KeywordInfo = proto.Field(
            proto.MESSAGE,
            number=1,
            message=criteria.KeywordInfo,
        )
        search_terms: MutableSequence[
            "Recommendation.KeywordRecommendation.SearchTerm"
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=4,
            message="Recommendation.KeywordRecommendation.SearchTerm",
        )
        recommended_cpc_bid_micros: int = proto.Field(
            proto.INT64,
            number=3,
            optional=True,
        )

    class TextAdRecommendation(proto.Message):
        r"""The text ad recommendation.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            ad (google.ads.googleads.v22.resources.types.Ad):
                Output only. Recommended ad.
            creation_date (str):
                Output only. Creation date of the recommended
                ad. YYYY-MM-DD format, for example, 2018-04-17.

                This field is a member of `oneof`_ ``_creation_date``.
            auto_apply_date (str):
                Output only. Date, if present, is the
                earliest when the recommendation will be auto
                applied. YYYY-MM-DD format, for example,
                2018-04-17.

                This field is a member of `oneof`_ ``_auto_apply_date``.
        """

        ad: gagr_ad.Ad = proto.Field(
            proto.MESSAGE,
            number=1,
            message=gagr_ad.Ad,
        )
        creation_date: str = proto.Field(
            proto.STRING,
            number=4,
            optional=True,
        )
        auto_apply_date: str = proto.Field(
            proto.STRING,
            number=5,
            optional=True,
        )

    class TargetCpaOptInRecommendation(proto.Message):
        r"""The Target CPA opt-in recommendation.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            options (MutableSequence[google.ads.googleads.v22.resources.types.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption]):
                Output only. The available goals and
                corresponding options for Target CPA strategy.
            recommended_target_cpa_micros (int):
                Output only. The recommended average CPA
                target. See required budget amount and impact of
                using this recommendation in options list.

                This field is a member of `oneof`_ ``_recommended_target_cpa_micros``.
        """

        class TargetCpaOptInRecommendationOption(proto.Message):
            r"""The Target CPA opt-in option with impact estimate.

            .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

            Attributes:
                goal (google.ads.googleads.v22.enums.types.TargetCpaOptInRecommendationGoalEnum.TargetCpaOptInRecommendationGoal):
                    Output only. The goal achieved by this
                    option.
                target_cpa_micros (int):
                    Output only. Average CPA target.

                    This field is a member of `oneof`_ ``_target_cpa_micros``.
                required_campaign_budget_amount_micros (int):
                    Output only. The minimum campaign budget, in
                    local currency for the account, required to
                    achieve the target CPA. Amount is specified in
                    micros, where one million is equivalent to one
                    currency unit.

                    This field is a member of `oneof`_ ``_required_campaign_budget_amount_micros``.
                impact (google.ads.googleads.v22.resources.types.Recommendation.RecommendationImpact):
                    Output only. The impact estimate if this
                    option is selected.
            """

            goal: (
                target_cpa_opt_in_recommendation_goal.TargetCpaOptInRecommendationGoalEnum.TargetCpaOptInRecommendationGoal
            ) = proto.Field(
                proto.ENUM,
                number=1,
                enum=target_cpa_opt_in_recommendation_goal.TargetCpaOptInRecommendationGoalEnum.TargetCpaOptInRecommendationGoal,
            )
            target_cpa_micros: int = proto.Field(
                proto.INT64,
                number=5,
                optional=True,
            )
            required_campaign_budget_amount_micros: int = proto.Field(
                proto.INT64,
                number=6,
                optional=True,
            )
            impact: "Recommendation.RecommendationImpact" = proto.Field(
                proto.MESSAGE,
                number=4,
                message="Recommendation.RecommendationImpact",
            )

        options: MutableSequence[
            "Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption"
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption",
        )
        recommended_target_cpa_micros: int = proto.Field(
            proto.INT64,
            number=3,
            optional=True,
        )

    class MaximizeConversionsOptInRecommendation(proto.Message):
        r"""The Maximize Conversions Opt-In recommendation.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            recommended_budget_amount_micros (int):
                Output only. The recommended new budget
                amount.

                This field is a member of `oneof`_ ``_recommended_budget_amount_micros``.
        """

        recommended_budget_amount_micros: int = proto.Field(
            proto.INT64,
            number=2,
            optional=True,
        )

    class EnhancedCpcOptInRecommendation(proto.Message):
        r"""The Enhanced Cost-Per-Click Opt-In recommendation."""

    class SearchPartnersOptInRecommendation(proto.Message):
        r"""The Search Partners Opt-In recommendation."""

    class MaximizeClicksOptInRecommendation(proto.Message):
        r"""The Maximize Clicks opt-in recommendation.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            recommended_budget_amount_micros (int):
                Output only. The recommended new budget
                amount. Only set if the current budget is too
                high.

                This field is a member of `oneof`_ ``_recommended_budget_amount_micros``.
        """

        recommended_budget_amount_micros: int = proto.Field(
            proto.INT64,
            number=2,
            optional=True,
        )

    class OptimizeAdRotationRecommendation(proto.Message):
        r"""The Optimize Ad Rotation recommendation."""

    class CalloutAssetRecommendation(proto.Message):
        r"""The callout asset recommendation.

        Attributes:
            recommended_campaign_callout_assets (MutableSequence[google.ads.googleads.v22.resources.types.Asset]):
                Output only. New callout extension assets
                recommended at the campaign level.
            recommended_customer_callout_assets (MutableSequence[google.ads.googleads.v22.resources.types.Asset]):
                Output only. New callout extension assets
                recommended at the customer level.
        """

        recommended_campaign_callout_assets: MutableSequence[asset.Asset] = (
            proto.RepeatedField(
                proto.MESSAGE,
                number=1,
                message=asset.Asset,
            )
        )
        recommended_customer_callout_assets: MutableSequence[asset.Asset] = (
            proto.RepeatedField(
                proto.MESSAGE,
                number=2,
                message=asset.Asset,
            )
        )

    class SitelinkAssetRecommendation(proto.Message):
        r"""The sitelink asset recommendation.

        Attributes:
            recommended_campaign_sitelink_assets (MutableSequence[google.ads.googleads.v22.resources.types.Asset]):
                Output only. New sitelink assets recommended
                at the campaign level.
            recommended_customer_sitelink_assets (MutableSequence[google.ads.googleads.v22.resources.types.Asset]):
                Output only. New sitelink assets recommended
                at the customer level.
        """

        recommended_campaign_sitelink_assets: MutableSequence[asset.Asset] = (
            proto.RepeatedField(
                proto.MESSAGE,
                number=1,
                message=asset.Asset,
            )
        )
        recommended_customer_sitelink_assets: MutableSequence[asset.Asset] = (
            proto.RepeatedField(
                proto.MESSAGE,
                number=2,
                message=asset.Asset,
            )
        )

    class CallAssetRecommendation(proto.Message):
        r"""The call asset recommendation."""

    class KeywordMatchTypeRecommendation(proto.Message):
        r"""The keyword match type recommendation.

        Attributes:
            keyword (google.ads.googleads.v22.common.types.KeywordInfo):
                Output only. The existing keyword where the
                match type should be more broad.
            recommended_match_type (google.ads.googleads.v22.enums.types.KeywordMatchTypeEnum.KeywordMatchType):
                Output only. The recommended new match type.
        """

        keyword: criteria.KeywordInfo = proto.Field(
            proto.MESSAGE,
            number=1,
            message=criteria.KeywordInfo,
        )
        recommended_match_type: (
            keyword_match_type.KeywordMatchTypeEnum.KeywordMatchType
        ) = proto.Field(
            proto.ENUM,
            number=2,
            enum=keyword_match_type.KeywordMatchTypeEnum.KeywordMatchType,
        )

    class MoveUnusedBudgetRecommendation(proto.Message):
        r"""The move unused budget recommendation.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            excess_campaign_budget (str):
                Output only. The excess budget's resource_name.

                This field is a member of `oneof`_ ``_excess_campaign_budget``.
            budget_recommendation (google.ads.googleads.v22.resources.types.Recommendation.CampaignBudgetRecommendation):
                Output only. The recommendation for the
                constrained budget to increase.
        """

        excess_campaign_budget: str = proto.Field(
            proto.STRING,
            number=3,
            optional=True,
        )
        budget_recommendation: "Recommendation.CampaignBudgetRecommendation" = (
            proto.Field(
                proto.MESSAGE,
                number=2,
                message="Recommendation.CampaignBudgetRecommendation",
            )
        )

    class TargetRoasOptInRecommendation(proto.Message):
        r"""The Target ROAS opt-in recommendation.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            recommended_target_roas (float):
                Output only. The recommended target ROAS
                (revenue per unit of spend). The value is
                between 0.01 and 1000.0, inclusive.

                This field is a member of `oneof`_ ``_recommended_target_roas``.
            required_campaign_budget_amount_micros (int):
                Output only. The minimum campaign budget, in
                local currency for the account, required to
                achieve the target ROAS. Amount is specified in
                micros, where one million is equivalent to one
                currency unit.

                This field is a member of `oneof`_ ``_required_campaign_budget_amount_micros``.
        """

        recommended_target_roas: float = proto.Field(
            proto.DOUBLE,
            number=1,
            optional=True,
        )
        required_campaign_budget_amount_micros: int = proto.Field(
            proto.INT64,
            number=2,
            optional=True,
        )

    class ResponsiveSearchAdAssetRecommendation(proto.Message):
        r"""The add responsive search ad asset recommendation.

        Attributes:
            current_ad (google.ads.googleads.v22.resources.types.Ad):
                Output only. The current ad to be updated.
            recommended_assets (google.ads.googleads.v22.resources.types.Ad):
                Output only. The recommended assets. This is
                populated only with the new headlines and/or
                descriptions, and is otherwise empty.
        """

        current_ad: gagr_ad.Ad = proto.Field(
            proto.MESSAGE,
            number=3,
            message=gagr_ad.Ad,
        )
        recommended_assets: gagr_ad.Ad = proto.Field(
            proto.MESSAGE,
            number=2,
            message=gagr_ad.Ad,
        )

    class ResponsiveSearchAdImproveAdStrengthRecommendation(proto.Message):
        r"""The responsive search ad improve ad strength recommendation.

        Attributes:
            current_ad (google.ads.googleads.v22.resources.types.Ad):
                Output only. The current ad to be updated.
            recommended_ad (google.ads.googleads.v22.resources.types.Ad):
                Output only. The updated ad.
        """

        current_ad: gagr_ad.Ad = proto.Field(
            proto.MESSAGE,
            number=1,
            message=gagr_ad.Ad,
        )
        recommended_ad: gagr_ad.Ad = proto.Field(
            proto.MESSAGE,
            number=2,
            message=gagr_ad.Ad,
        )

    class ResponsiveSearchAdRecommendation(proto.Message):
        r"""The add responsive search ad recommendation.

        Attributes:
            ad (google.ads.googleads.v22.resources.types.Ad):
                Output only. Recommended ad.
        """

        ad: gagr_ad.Ad = proto.Field(
            proto.MESSAGE,
            number=1,
            message=gagr_ad.Ad,
        )

    class UseBroadMatchKeywordRecommendation(proto.Message):
        r"""The use broad match keyword recommendation.

        Attributes:
            keyword (MutableSequence[google.ads.googleads.v22.common.types.KeywordInfo]):
                Output only. Sample of keywords to be
                expanded to Broad Match.
            suggested_keywords_count (int):
                Output only. Total number of keywords to be
                expanded to Broad Match in the campaign.
            campaign_keywords_count (int):
                Output only. Total number of keywords in the
                campaign.
            campaign_uses_shared_budget (bool):
                Output only. Whether the associated campaign
                uses a shared budget.
            required_campaign_budget_amount_micros (int):
                Output only. The budget recommended to avoid
                becoming budget constrained after applying the
                recommendation.
        """

        keyword: MutableSequence[criteria.KeywordInfo] = proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=criteria.KeywordInfo,
        )
        suggested_keywords_count: int = proto.Field(
            proto.INT64,
            number=2,
        )
        campaign_keywords_count: int = proto.Field(
            proto.INT64,
            number=3,
        )
        campaign_uses_shared_budget: bool = proto.Field(
            proto.BOOL,
            number=4,
        )
        required_campaign_budget_amount_micros: int = proto.Field(
            proto.INT64,
            number=5,
        )

    class UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation(
        proto.Message
    ):
        r"""The upgrade a Smart Shopping campaign to a Performance Max
        campaign recommendation.

        Attributes:
            merchant_id (int):
                Output only. ID of Merchant Center account.
            sales_country_code (str):
                Output only. Country whose products from
                merchant's inventory should be included.
        """

        merchant_id: int = proto.Field(
            proto.INT64,
            number=1,
        )
        sales_country_code: str = proto.Field(
            proto.STRING,
            number=2,
        )

    class RaiseTargetCpaBidTooLowRecommendation(proto.Message):
        r"""The raise target CPA bid too low recommendation.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            recommended_target_multiplier (float):
                Output only. A number greater than 1.0
                indicating the factor by which we recommend the
                target CPA should be increased.

                This field is a member of `oneof`_ ``_recommended_target_multiplier``.
            average_target_cpa_micros (int):
                Output only. The current average target CPA
                of the campaign, in micros of customer local
                currency.

                This field is a member of `oneof`_ ``_average_target_cpa_micros``.
        """

        recommended_target_multiplier: float = proto.Field(
            proto.DOUBLE,
            number=1,
            optional=True,
        )
        average_target_cpa_micros: int = proto.Field(
            proto.INT64,
            number=2,
            optional=True,
        )

    class DisplayExpansionOptInRecommendation(proto.Message):
        r"""The Display Expansion opt-in recommendation."""

    class UpgradeLocalCampaignToPerformanceMaxRecommendation(proto.Message):
        r"""The Upgrade Local campaign to Performance Max campaign
        recommendation.

        """

    class ForecastingSetTargetRoasRecommendation(proto.Message):
        r"""The forecasting set target ROAS recommendation.

        Attributes:
            recommended_target_roas (float):
                Output only. The recommended target ROAS
                (revenue per unit of spend). The value is
                between 0.01 and 1000.0, inclusive.
            campaign_budget (google.ads.googleads.v22.resources.types.Recommendation.CampaignBudget):
                Output only. The campaign budget.
        """

        recommended_target_roas: float = proto.Field(
            proto.DOUBLE,
            number=1,
        )
        campaign_budget: "Recommendation.CampaignBudget" = proto.Field(
            proto.MESSAGE,
            number=2,
            message="Recommendation.CampaignBudget",
        )

    class ShoppingOfferAttributeRecommendation(proto.Message):
        r"""The shopping recommendation to add an attribute to offers
        that are demoted because it is missing.

        Attributes:
            merchant (google.ads.googleads.v22.resources.types.Recommendation.MerchantInfo):
                Output only. The details of the Merchant
                Center account.
            feed_label (str):
                Output only. The campaign feed label.
            offers_count (int):
                Output only. The number of online, servable
                offers.
            demoted_offers_count (int):
                Output only. The number of online, servable
                offers that are demoted for missing attributes.
                Visit the Merchant Center for more details.
        """

        merchant: "Recommendation.MerchantInfo" = proto.Field(
            proto.MESSAGE,
            number=1,
            message="Recommendation.MerchantInfo",
        )
        feed_label: str = proto.Field(
            proto.STRING,
            number=2,
        )
        offers_count: int = proto.Field(
            proto.INT64,
            number=3,
        )
        demoted_offers_count: int = proto.Field(
            proto.INT64,
            number=4,
        )

    class ShoppingFixDisapprovedProductsRecommendation(proto.Message):
        r"""The shopping recommendation to fix disapproved products in a
        Shopping Campaign Inventory.

        Attributes:
            merchant (google.ads.googleads.v22.resources.types.Recommendation.MerchantInfo):
                Output only. The details of the Merchant
                Center account.
            feed_label (str):
                Output only. The feed label for the campaign.
            products_count (int):
                Output only. The number of products of the
                campaign.
            disapproved_products_count (int):
                Output only. The numbers of products of the
                campaign that are disapproved.
        """

        merchant: "Recommendation.MerchantInfo" = proto.Field(
            proto.MESSAGE,
            number=1,
            message="Recommendation.MerchantInfo",
        )
        feed_label: str = proto.Field(
            proto.STRING,
            number=2,
        )
        products_count: int = proto.Field(
            proto.INT64,
            number=3,
        )
        disapproved_products_count: int = proto.Field(
            proto.INT64,
            number=4,
        )

    class ShoppingTargetAllOffersRecommendation(proto.Message):
        r"""The shopping recommendation to create a catch-all campaign
        that targets all offers.

        Attributes:
            merchant (google.ads.googleads.v22.resources.types.Recommendation.MerchantInfo):
                Output only. The details of the Merchant
                Center account.
            untargeted_offers_count (int):
                Output only. The number of untargeted offers.
            feed_label (str):
                Output only. The offer feed label.
        """

        merchant: "Recommendation.MerchantInfo" = proto.Field(
            proto.MESSAGE,
            number=1,
            message="Recommendation.MerchantInfo",
        )
        untargeted_offers_count: int = proto.Field(
            proto.INT64,
            number=2,
        )
        feed_label: str = proto.Field(
            proto.STRING,
            number=3,
        )

    class ShoppingAddProductsToCampaignRecommendation(proto.Message):
        r"""The shopping recommendation to add products to a Shopping
        Campaign Inventory.

        Attributes:
            merchant (google.ads.googleads.v22.resources.types.Recommendation.MerchantInfo):
                Output only. The details of the Merchant
                Center account.
            feed_label (str):
                Output only. The feed label for the campaign.
            reason (google.ads.googleads.v22.enums.types.ShoppingAddProductsToCampaignRecommendationEnum.Reason):
                Output only. The reason why no products are
                attached to the campaign.
        """

        merchant: "Recommendation.MerchantInfo" = proto.Field(
            proto.MESSAGE,
            number=1,
            message="Recommendation.MerchantInfo",
        )
        feed_label: str = proto.Field(
            proto.STRING,
            number=2,
        )
        reason: (
            shopping_add_products_to_campaign_recommendation_enum.ShoppingAddProductsToCampaignRecommendationEnum.Reason
        ) = proto.Field(
            proto.ENUM,
            number=3,
            enum=shopping_add_products_to_campaign_recommendation_enum.ShoppingAddProductsToCampaignRecommendationEnum.Reason,
        )

    class ShoppingMerchantCenterAccountSuspensionRecommendation(proto.Message):
        r"""The shopping recommendation to fix Merchant Center account
        suspension issues.

        Attributes:
            merchant (google.ads.googleads.v22.resources.types.Recommendation.MerchantInfo):
                Output only. The details of the Merchant
                Center account.
            feed_label (str):
                Output only. The feed label of the campaign
                for which the suspension happened.
        """

        merchant: "Recommendation.MerchantInfo" = proto.Field(
            proto.MESSAGE,
            number=1,
            message="Recommendation.MerchantInfo",
        )
        feed_label: str = proto.Field(
            proto.STRING,
            number=2,
        )

    class ShoppingMigrateRegularShoppingCampaignOffersToPerformanceMaxRecommendation(
        proto.Message
    ):
        r"""The shopping recommendation to migrate Regular Shopping
        Campaign targeted offers to Performance Max campaigns.

        Attributes:
            merchant (google.ads.googleads.v22.resources.types.Recommendation.MerchantInfo):
                Output only. The details of the Merchant
                Center account.
            feed_label (str):
                Output only. The feed label of the offers
                targeted by the campaigns sharing this
                suggestion.
        """

        merchant: "Recommendation.MerchantInfo" = proto.Field(
            proto.MESSAGE,
            number=1,
            message="Recommendation.MerchantInfo",
        )
        feed_label: str = proto.Field(
            proto.STRING,
            number=2,
        )

    class TargetAdjustmentInfo(proto.Message):
        r"""Information of a target adjustment recommendation.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            shared_set (str):
                Output only. The shared set resource name of
                the portfolio bidding strategy where the target
                is defined. Only populated if the recommendation
                is portfolio level.

                This field is a member of `oneof`_ ``_shared_set``.
            recommended_target_multiplier (float):
                Output only. The factor by which we recommend
                the target to be adjusted by.
            current_average_target_micros (int):
                Output only. The current average target of
                the campaign or portfolio targeted by this
                recommendation.
        """

        shared_set: str = proto.Field(
            proto.STRING,
            number=1,
            optional=True,
        )
        recommended_target_multiplier: float = proto.Field(
            proto.DOUBLE,
            number=2,
        )
        current_average_target_micros: int = proto.Field(
            proto.INT64,
            number=3,
        )

    class RaiseTargetCpaRecommendation(proto.Message):
        r"""Recommendation to raise Target CPA.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            target_adjustment (google.ads.googleads.v22.resources.types.Recommendation.TargetAdjustmentInfo):
                Output only. The relevant information
                describing the recommended target adjustment.
            app_bidding_goal (google.ads.googleads.v22.enums.types.AppBiddingGoalEnum.AppBiddingGoal):
                Output only. Represents the goal towards
                which the bidding strategy should optimize. Only
                populated for App Campaigns.

                This field is a member of `oneof`_ ``_app_bidding_goal``.
        """

        target_adjustment: "Recommendation.TargetAdjustmentInfo" = proto.Field(
            proto.MESSAGE,
            number=1,
            message="Recommendation.TargetAdjustmentInfo",
        )
        app_bidding_goal: (
            gage_app_bidding_goal.AppBiddingGoalEnum.AppBiddingGoal
        ) = proto.Field(
            proto.ENUM,
            number=2,
            optional=True,
            enum=gage_app_bidding_goal.AppBiddingGoalEnum.AppBiddingGoal,
        )

    class LowerTargetRoasRecommendation(proto.Message):
        r"""Recommendation to lower Target ROAS.

        Attributes:
            target_adjustment (google.ads.googleads.v22.resources.types.Recommendation.TargetAdjustmentInfo):
                Output only. The relevant information
                describing the recommended target adjustment.
        """

        target_adjustment: "Recommendation.TargetAdjustmentInfo" = proto.Field(
            proto.MESSAGE,
            number=1,
            message="Recommendation.TargetAdjustmentInfo",
        )

    class DynamicImageExtensionOptInRecommendation(proto.Message):
        r"""Recommendation to enable dynamic image extensions on the
        account, allowing Google to find the best images from ad landing
        pages and complement text ads.

        """

    class CampaignBudget(proto.Message):
        r"""A campaign budget shared amongst various budget
        recommendation types.

        Attributes:
            current_amount_micros (int):
                Output only. Current budget amount.
            recommended_new_amount_micros (int):
                Output only. Recommended budget amount.
            new_start_date (str):
                Output only. The date when the new budget would start being
                used. This field will be set for the following
                recommendation types: FORECASTING_SET_TARGET_ROAS ,
                FORECASTING_SET_TARGET_CPA YYYY-MM-DD format, for example,
                2018-04-17.
        """

        current_amount_micros: int = proto.Field(
            proto.INT64,
            number=1,
        )
        recommended_new_amount_micros: int = proto.Field(
            proto.INT64,
            number=2,
        )
        new_start_date: str = proto.Field(
            proto.STRING,
            number=3,
        )

    class PerformanceMaxOptInRecommendation(proto.Message):
        r"""The Performance Max Opt In recommendation."""

    class ImprovePerformanceMaxAdStrengthRecommendation(proto.Message):
        r"""Recommendation to improve the asset group strength of a
        Performance Max campaign to an "Excellent" rating.

        Attributes:
            asset_group (str):
                Output only. The asset group resource name.
            ad_strength (google.ads.googleads.v22.enums.types.AdStrengthEnum.AdStrength):
                Output only. The current ad strength score of
                the asset group.
        """

        asset_group: str = proto.Field(
            proto.STRING,
            number=1,
        )
        ad_strength: gage_ad_strength.AdStrengthEnum.AdStrength = proto.Field(
            proto.ENUM,
            number=2,
            enum=gage_ad_strength.AdStrengthEnum.AdStrength,
        )

    class MigrateDynamicSearchAdsCampaignToPerformanceMaxRecommendation(
        proto.Message
    ):
        r"""The Dynamic Search Ads to Performance Max migration
        recommendation.

        Attributes:
            apply_link (str):
                Output only. A link to the Google Ads UI
                where the customer can manually apply the
                recommendation.
        """

        apply_link: str = proto.Field(
            proto.STRING,
            number=1,
        )

    class ForecastingSetTargetCpaRecommendation(proto.Message):
        r"""The set target CPA recommendations.

        Attributes:
            recommended_target_cpa_micros (int):
                Output only. The recommended target CPA.
            campaign_budget (google.ads.googleads.v22.resources.types.Recommendation.CampaignBudget):
                Output only. The campaign budget.
        """

        recommended_target_cpa_micros: int = proto.Field(
            proto.INT64,
            number=1,
        )
        campaign_budget: "Recommendation.CampaignBudget" = proto.Field(
            proto.MESSAGE,
            number=2,
            message="Recommendation.CampaignBudget",
        )

    class MaximizeConversionValueOptInRecommendation(proto.Message):
        r"""Recommendation to opt into Maximize Conversion Value bidding
        strategy.

        """

    class ImproveGoogleTagCoverageRecommendation(proto.Message):
        r"""Recommendation to deploy Google Tag on more pages."""

    class PerformanceMaxFinalUrlOptInRecommendation(proto.Message):
        r"""Recommendation to turn on Final URL expansion for your
        Performance Max campaigns.

        """

    class RefreshCustomerMatchListRecommendation(proto.Message):
        r"""The recommendation to update a customer list that hasn't been
        updated in the last 90 days. The customer receiving the
        recommendation is not necessarily the owner account. The owner
        account should update the customer list.

        Attributes:
            user_list_id (int):
                Output only. The user list ID.
            user_list_name (str):
                Output only. The name of the list.
            days_since_last_refresh (int):
                Output only. Days since last refresh.
            top_spending_account (MutableSequence[google.ads.googleads.v22.resources.types.Recommendation.AccountInfo]):
                Output only. The top spending account.
            targeting_accounts_count (int):
                Output only. User lists can be shared with other accounts by
                the owner. targeting_accounts_count is the number of those
                accounts that can use it for targeting.
            owner_account (google.ads.googleads.v22.resources.types.Recommendation.AccountInfo):
                Output only. The owner account. This is the
                account that should update the customer list.
        """

        user_list_id: int = proto.Field(
            proto.INT64,
            number=1,
        )
        user_list_name: str = proto.Field(
            proto.STRING,
            number=2,
        )
        days_since_last_refresh: int = proto.Field(
            proto.INT64,
            number=3,
        )
        top_spending_account: MutableSequence["Recommendation.AccountInfo"] = (
            proto.RepeatedField(
                proto.MESSAGE,
                number=4,
                message="Recommendation.AccountInfo",
            )
        )
        targeting_accounts_count: int = proto.Field(
            proto.INT64,
            number=5,
        )
        owner_account: "Recommendation.AccountInfo" = proto.Field(
            proto.MESSAGE,
            number=6,
            message="Recommendation.AccountInfo",
        )

    class AccountInfo(proto.Message):
        r"""Wrapper for information about a Google Ads account.

        Attributes:
            customer_id (int):
                Output only. The customer ID of the account.
            descriptive_name (str):
                Output only. The descriptive name of the
                account.
        """

        customer_id: int = proto.Field(
            proto.INT64,
            number=1,
        )
        descriptive_name: str = proto.Field(
            proto.STRING,
            number=2,
        )

    class CustomAudienceOptInRecommendation(proto.Message):
        r"""The Custom Audience Opt In recommendation.

        Attributes:
            keywords (MutableSequence[google.ads.googleads.v22.common.types.KeywordInfo]):
                Output only. The list of keywords to use for
                custom audience creation.
        """

        keywords: MutableSequence[criteria.KeywordInfo] = proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=criteria.KeywordInfo,
        )

    class LeadFormAssetRecommendation(proto.Message):
        r"""The lead form asset recommendation."""

    class ImproveDemandGenAdStrengthRecommendation(proto.Message):
        r"""The improve Demand Gen ad strength recommendation.

        Attributes:
            ad (str):
                Output only. The resource name of the ad that
                can be improved.
            ad_strength (google.ads.googleads.v22.enums.types.AdStrengthEnum.AdStrength):
                Output only. The current ad strength.
            demand_gen_asset_action_items (MutableSequence[str]):
                Output only. A list of recommendations to
                improve the ad strength.
        """

        ad: str = proto.Field(
            proto.STRING,
            number=1,
        )
        ad_strength: gage_ad_strength.AdStrengthEnum.AdStrength = proto.Field(
            proto.ENUM,
            number=2,
            enum=gage_ad_strength.AdStrengthEnum.AdStrength,
        )
        demand_gen_asset_action_items: MutableSequence[str] = (
            proto.RepeatedField(
                proto.STRING,
                number=3,
            )
        )

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    type_: recommendation_type.RecommendationTypeEnum.RecommendationType = (
        proto.Field(
            proto.ENUM,
            number=2,
            enum=recommendation_type.RecommendationTypeEnum.RecommendationType,
        )
    )
    impact: RecommendationImpact = proto.Field(
        proto.MESSAGE,
        number=3,
        message=RecommendationImpact,
    )
    campaign_budget: str = proto.Field(
        proto.STRING,
        number=24,
        optional=True,
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=25,
        optional=True,
    )
    ad_group: str = proto.Field(
        proto.STRING,
        number=26,
        optional=True,
    )
    dismissed: bool = proto.Field(
        proto.BOOL,
        number=27,
        optional=True,
    )
    campaigns: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=38,
    )
    campaign_budget_recommendation: CampaignBudgetRecommendation = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="recommendation",
        message=CampaignBudgetRecommendation,
    )
    forecasting_campaign_budget_recommendation: CampaignBudgetRecommendation = (
        proto.Field(
            proto.MESSAGE,
            number=22,
            oneof="recommendation",
            message=CampaignBudgetRecommendation,
        )
    )
    keyword_recommendation: KeywordRecommendation = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="recommendation",
        message=KeywordRecommendation,
    )
    text_ad_recommendation: TextAdRecommendation = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="recommendation",
        message=TextAdRecommendation,
    )
    target_cpa_opt_in_recommendation: TargetCpaOptInRecommendation = (
        proto.Field(
            proto.MESSAGE,
            number=10,
            oneof="recommendation",
            message=TargetCpaOptInRecommendation,
        )
    )
    maximize_conversions_opt_in_recommendation: (
        MaximizeConversionsOptInRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=11,
        oneof="recommendation",
        message=MaximizeConversionsOptInRecommendation,
    )
    enhanced_cpc_opt_in_recommendation: EnhancedCpcOptInRecommendation = (
        proto.Field(
            proto.MESSAGE,
            number=12,
            oneof="recommendation",
            message=EnhancedCpcOptInRecommendation,
        )
    )
    search_partners_opt_in_recommendation: SearchPartnersOptInRecommendation = (
        proto.Field(
            proto.MESSAGE,
            number=14,
            oneof="recommendation",
            message=SearchPartnersOptInRecommendation,
        )
    )
    maximize_clicks_opt_in_recommendation: MaximizeClicksOptInRecommendation = (
        proto.Field(
            proto.MESSAGE,
            number=15,
            oneof="recommendation",
            message=MaximizeClicksOptInRecommendation,
        )
    )
    optimize_ad_rotation_recommendation: OptimizeAdRotationRecommendation = (
        proto.Field(
            proto.MESSAGE,
            number=16,
            oneof="recommendation",
            message=OptimizeAdRotationRecommendation,
        )
    )
    keyword_match_type_recommendation: KeywordMatchTypeRecommendation = (
        proto.Field(
            proto.MESSAGE,
            number=20,
            oneof="recommendation",
            message=KeywordMatchTypeRecommendation,
        )
    )
    move_unused_budget_recommendation: MoveUnusedBudgetRecommendation = (
        proto.Field(
            proto.MESSAGE,
            number=21,
            oneof="recommendation",
            message=MoveUnusedBudgetRecommendation,
        )
    )
    target_roas_opt_in_recommendation: TargetRoasOptInRecommendation = (
        proto.Field(
            proto.MESSAGE,
            number=23,
            oneof="recommendation",
            message=TargetRoasOptInRecommendation,
        )
    )
    responsive_search_ad_recommendation: ResponsiveSearchAdRecommendation = (
        proto.Field(
            proto.MESSAGE,
            number=28,
            oneof="recommendation",
            message=ResponsiveSearchAdRecommendation,
        )
    )
    marginal_roi_campaign_budget_recommendation: (
        CampaignBudgetRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=29,
        oneof="recommendation",
        message=CampaignBudgetRecommendation,
    )
    use_broad_match_keyword_recommendation: (
        UseBroadMatchKeywordRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=30,
        oneof="recommendation",
        message=UseBroadMatchKeywordRecommendation,
    )
    responsive_search_ad_asset_recommendation: (
        ResponsiveSearchAdAssetRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=31,
        oneof="recommendation",
        message=ResponsiveSearchAdAssetRecommendation,
    )
    upgrade_smart_shopping_campaign_to_performance_max_recommendation: (
        UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=32,
        oneof="recommendation",
        message=UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation,
    )
    responsive_search_ad_improve_ad_strength_recommendation: (
        ResponsiveSearchAdImproveAdStrengthRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=33,
        oneof="recommendation",
        message=ResponsiveSearchAdImproveAdStrengthRecommendation,
    )
    display_expansion_opt_in_recommendation: (
        DisplayExpansionOptInRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=34,
        oneof="recommendation",
        message=DisplayExpansionOptInRecommendation,
    )
    upgrade_local_campaign_to_performance_max_recommendation: (
        UpgradeLocalCampaignToPerformanceMaxRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=35,
        oneof="recommendation",
        message=UpgradeLocalCampaignToPerformanceMaxRecommendation,
    )
    raise_target_cpa_bid_too_low_recommendation: (
        RaiseTargetCpaBidTooLowRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=36,
        oneof="recommendation",
        message=RaiseTargetCpaBidTooLowRecommendation,
    )
    forecasting_set_target_roas_recommendation: (
        ForecastingSetTargetRoasRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=37,
        oneof="recommendation",
        message=ForecastingSetTargetRoasRecommendation,
    )
    callout_asset_recommendation: CalloutAssetRecommendation = proto.Field(
        proto.MESSAGE,
        number=39,
        oneof="recommendation",
        message=CalloutAssetRecommendation,
    )
    sitelink_asset_recommendation: SitelinkAssetRecommendation = proto.Field(
        proto.MESSAGE,
        number=40,
        oneof="recommendation",
        message=SitelinkAssetRecommendation,
    )
    call_asset_recommendation: CallAssetRecommendation = proto.Field(
        proto.MESSAGE,
        number=41,
        oneof="recommendation",
        message=CallAssetRecommendation,
    )
    shopping_add_age_group_recommendation: (
        ShoppingOfferAttributeRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=42,
        oneof="recommendation",
        message=ShoppingOfferAttributeRecommendation,
    )
    shopping_add_color_recommendation: ShoppingOfferAttributeRecommendation = (
        proto.Field(
            proto.MESSAGE,
            number=43,
            oneof="recommendation",
            message=ShoppingOfferAttributeRecommendation,
        )
    )
    shopping_add_gender_recommendation: ShoppingOfferAttributeRecommendation = (
        proto.Field(
            proto.MESSAGE,
            number=44,
            oneof="recommendation",
            message=ShoppingOfferAttributeRecommendation,
        )
    )
    shopping_add_gtin_recommendation: ShoppingOfferAttributeRecommendation = (
        proto.Field(
            proto.MESSAGE,
            number=45,
            oneof="recommendation",
            message=ShoppingOfferAttributeRecommendation,
        )
    )
    shopping_add_more_identifiers_recommendation: (
        ShoppingOfferAttributeRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=46,
        oneof="recommendation",
        message=ShoppingOfferAttributeRecommendation,
    )
    shopping_add_size_recommendation: ShoppingOfferAttributeRecommendation = (
        proto.Field(
            proto.MESSAGE,
            number=47,
            oneof="recommendation",
            message=ShoppingOfferAttributeRecommendation,
        )
    )
    shopping_add_products_to_campaign_recommendation: (
        ShoppingAddProductsToCampaignRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=48,
        oneof="recommendation",
        message=ShoppingAddProductsToCampaignRecommendation,
    )
    shopping_fix_disapproved_products_recommendation: (
        ShoppingFixDisapprovedProductsRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=49,
        oneof="recommendation",
        message=ShoppingFixDisapprovedProductsRecommendation,
    )
    shopping_target_all_offers_recommendation: (
        ShoppingTargetAllOffersRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=50,
        oneof="recommendation",
        message=ShoppingTargetAllOffersRecommendation,
    )
    shopping_fix_suspended_merchant_center_account_recommendation: (
        ShoppingMerchantCenterAccountSuspensionRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=51,
        oneof="recommendation",
        message=ShoppingMerchantCenterAccountSuspensionRecommendation,
    )
    shopping_fix_merchant_center_account_suspension_warning_recommendation: (
        ShoppingMerchantCenterAccountSuspensionRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=52,
        oneof="recommendation",
        message=ShoppingMerchantCenterAccountSuspensionRecommendation,
    )
    shopping_migrate_regular_shopping_campaign_offers_to_performance_max_recommendation: ShoppingMigrateRegularShoppingCampaignOffersToPerformanceMaxRecommendation = proto.Field(
        proto.MESSAGE,
        number=53,
        oneof="recommendation",
        message=ShoppingMigrateRegularShoppingCampaignOffersToPerformanceMaxRecommendation,
    )
    dynamic_image_extension_opt_in_recommendation: (
        DynamicImageExtensionOptInRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=54,
        oneof="recommendation",
        message=DynamicImageExtensionOptInRecommendation,
    )
    raise_target_cpa_recommendation: RaiseTargetCpaRecommendation = proto.Field(
        proto.MESSAGE,
        number=55,
        oneof="recommendation",
        message=RaiseTargetCpaRecommendation,
    )
    lower_target_roas_recommendation: LowerTargetRoasRecommendation = (
        proto.Field(
            proto.MESSAGE,
            number=56,
            oneof="recommendation",
            message=LowerTargetRoasRecommendation,
        )
    )
    performance_max_opt_in_recommendation: PerformanceMaxOptInRecommendation = (
        proto.Field(
            proto.MESSAGE,
            number=57,
            oneof="recommendation",
            message=PerformanceMaxOptInRecommendation,
        )
    )
    improve_performance_max_ad_strength_recommendation: (
        ImprovePerformanceMaxAdStrengthRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=58,
        oneof="recommendation",
        message=ImprovePerformanceMaxAdStrengthRecommendation,
    )
    migrate_dynamic_search_ads_campaign_to_performance_max_recommendation: (
        MigrateDynamicSearchAdsCampaignToPerformanceMaxRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=59,
        oneof="recommendation",
        message=MigrateDynamicSearchAdsCampaignToPerformanceMaxRecommendation,
    )
    forecasting_set_target_cpa_recommendation: (
        ForecastingSetTargetCpaRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=60,
        oneof="recommendation",
        message=ForecastingSetTargetCpaRecommendation,
    )
    set_target_cpa_recommendation: ForecastingSetTargetCpaRecommendation = (
        proto.Field(
            proto.MESSAGE,
            number=61,
            oneof="recommendation",
            message=ForecastingSetTargetCpaRecommendation,
        )
    )
    set_target_roas_recommendation: ForecastingSetTargetRoasRecommendation = (
        proto.Field(
            proto.MESSAGE,
            number=62,
            oneof="recommendation",
            message=ForecastingSetTargetRoasRecommendation,
        )
    )
    maximize_conversion_value_opt_in_recommendation: (
        MaximizeConversionValueOptInRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=63,
        oneof="recommendation",
        message=MaximizeConversionValueOptInRecommendation,
    )
    improve_google_tag_coverage_recommendation: (
        ImproveGoogleTagCoverageRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=64,
        oneof="recommendation",
        message=ImproveGoogleTagCoverageRecommendation,
    )
    performance_max_final_url_opt_in_recommendation: (
        PerformanceMaxFinalUrlOptInRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=65,
        oneof="recommendation",
        message=PerformanceMaxFinalUrlOptInRecommendation,
    )
    refresh_customer_match_list_recommendation: (
        RefreshCustomerMatchListRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=66,
        oneof="recommendation",
        message=RefreshCustomerMatchListRecommendation,
    )
    custom_audience_opt_in_recommendation: CustomAudienceOptInRecommendation = (
        proto.Field(
            proto.MESSAGE,
            number=67,
            oneof="recommendation",
            message=CustomAudienceOptInRecommendation,
        )
    )
    lead_form_asset_recommendation: LeadFormAssetRecommendation = proto.Field(
        proto.MESSAGE,
        number=68,
        oneof="recommendation",
        message=LeadFormAssetRecommendation,
    )
    improve_demand_gen_ad_strength_recommendation: (
        ImproveDemandGenAdStrengthRecommendation
    ) = proto.Field(
        proto.MESSAGE,
        number=69,
        oneof="recommendation",
        message=ImproveDemandGenAdStrengthRecommendation,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.common.types import criteria
from google.ads.googleads.v20.enums.types import (
    ad_destination_type as gage_ad_destination_type,
)
from google.ads.googleads.v20.enums.types import (
    ad_format_type as gage_ad_format_type,
)
from google.ads.googleads.v20.enums.types import (
    ad_network_type as gage_ad_network_type,
)
from google.ads.googleads.v20.enums.types import age_range_type
from google.ads.googleads.v20.enums.types import (
    budget_campaign_association_status as gage_budget_campaign_association_status,
)
from google.ads.googleads.v20.enums.types import click_type as gage_click_type
from google.ads.googleads.v20.enums.types import (
    conversion_action_category as gage_conversion_action_category,
)
from google.ads.googleads.v20.enums.types import (
    conversion_attribution_event_type as gage_conversion_attribution_event_type,
)
from google.ads.googleads.v20.enums.types import (
    conversion_lag_bucket as gage_conversion_lag_bucket,
)
from google.ads.googleads.v20.enums.types import (
    conversion_or_adjustment_lag_bucket as gage_conversion_or_adjustment_lag_bucket,
)
from google.ads.googleads.v20.enums.types import (
    conversion_value_rule_primary_dimension as gage_conversion_value_rule_primary_dimension,
)
from google.ads.googleads.v20.enums.types import (
    converting_user_prior_engagement_type_and_ltv_bucket,
)
from google.ads.googleads.v20.enums.types import day_of_week as gage_day_of_week
from google.ads.googleads.v20.enums.types import device as gage_device
from google.ads.googleads.v20.enums.types import (
    external_conversion_source as gage_external_conversion_source,
)
from google.ads.googleads.v20.enums.types import gender_type
from google.ads.googleads.v20.enums.types import (
    hotel_date_selection_type as gage_hotel_date_selection_type,
)
from google.ads.googleads.v20.enums.types import (
    hotel_price_bucket as gage_hotel_price_bucket,
)
from google.ads.googleads.v20.enums.types import (
    hotel_rate_type as gage_hotel_rate_type,
)
from google.ads.googleads.v20.enums.types import (
    month_of_year as gage_month_of_year,
)
from google.ads.googleads.v20.enums.types import (
    product_channel as gage_product_channel,
)
from google.ads.googleads.v20.enums.types import (
    product_channel_exclusivity as gage_product_channel_exclusivity,
)
from google.ads.googleads.v20.enums.types import (
    product_condition as gage_product_condition,
)
from google.ads.googleads.v20.enums.types import (
    recommendation_type as gage_recommendation_type,
)
from google.ads.googleads.v20.enums.types import (
    search_engine_results_page_type as gage_search_engine_results_page_type,
)
from google.ads.googleads.v20.enums.types import (
    search_term_match_type as gage_search_term_match_type,
)
from google.ads.googleads.v20.enums.types import (
    sk_ad_network_ad_event_type as gage_sk_ad_network_ad_event_type,
)
from google.ads.googleads.v20.enums.types import (
    sk_ad_network_attribution_credit as gage_sk_ad_network_attribution_credit,
)
from google.ads.googleads.v20.enums.types import (
    sk_ad_network_coarse_conversion_value as gage_sk_ad_network_coarse_conversion_value,
)
from google.ads.googleads.v20.enums.types import (
    sk_ad_network_source_type as gage_sk_ad_network_source_type,
)
from google.ads.googleads.v20.enums.types import (
    sk_ad_network_user_type as gage_sk_ad_network_user_type,
)
from google.ads.googleads.v20.enums.types import slot as gage_slot


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.common",
    marshal="google.ads.googleads.v20",
    manifest={
        "Segments",
        "Keyword",
        "BudgetCampaignAssociationStatus",
        "AssetInteractionTarget",
        "SkAdNetworkSourceApp",
    },
)


class Segments(proto.Message):
    r"""Segment only fields.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        activity_account_id (int):
            Activity account ID.

            This field is a member of `oneof`_ ``_activity_account_id``.
        activity_city (str):
            The city where the travel activity is
            available.

            This field is a member of `oneof`_ ``_activity_city``.
        activity_country (str):
            The country where the travel activity is
            available.

            This field is a member of `oneof`_ ``_activity_country``.
        activity_rating (int):
            Activity rating.

            This field is a member of `oneof`_ ``_activity_rating``.
        activity_state (str):
            The state where the travel activity is
            available.

            This field is a member of `oneof`_ ``_activity_state``.
        external_activity_id (str):
            Advertiser supplied activity ID.

            This field is a member of `oneof`_ ``_external_activity_id``.
        ad_destination_type (google.ads.googleads.v20.enums.types.AdDestinationTypeEnum.AdDestinationType):
            Ad Destination type.
        ad_format_type (google.ads.googleads.v20.enums.types.AdFormatTypeEnum.AdFormatType):
            Ad Format type.
        ad_network_type (google.ads.googleads.v20.enums.types.AdNetworkTypeEnum.AdNetworkType):
            Ad network type.
        ad_group (str):
            Resource name of the ad group.

            This field is a member of `oneof`_ ``_ad_group``.
        asset_group (str):
            Resource name of the asset group.

            This field is a member of `oneof`_ ``_asset_group``.
        auction_insight_domain (str):
            Domain (visible URL) of a participant in the
            Auction Insights report.

            This field is a member of `oneof`_ ``_auction_insight_domain``.
        budget_campaign_association_status (google.ads.googleads.v20.common.types.BudgetCampaignAssociationStatus):
            Budget campaign association status.
        campaign (str):
            Resource name of the campaign.

            This field is a member of `oneof`_ ``_campaign``.
        click_type (google.ads.googleads.v20.enums.types.ClickTypeEnum.ClickType):
            Click type.
        conversion_action (str):
            Resource name of the conversion action.

            This field is a member of `oneof`_ ``_conversion_action``.
        conversion_action_category (google.ads.googleads.v20.enums.types.ConversionActionCategoryEnum.ConversionActionCategory):
            Conversion action category.
        conversion_action_name (str):
            Conversion action name.

            This field is a member of `oneof`_ ``_conversion_action_name``.
        conversion_adjustment (bool):
            This segments your conversion columns by the
            original conversion and conversion value versus
            the delta if conversions were adjusted. False
            row has the data as originally stated; While
            true row has the delta between data now and the
            data as originally stated. Summing the two
            together results post-adjustment data.

            This field is a member of `oneof`_ ``_conversion_adjustment``.
        conversion_attribution_event_type (google.ads.googleads.v20.enums.types.ConversionAttributionEventTypeEnum.ConversionAttributionEventType):
            Conversion attribution event type.
        conversion_lag_bucket (google.ads.googleads.v20.enums.types.ConversionLagBucketEnum.ConversionLagBucket):
            An enum value representing the number of days
            between the impression and the conversion.
        conversion_or_adjustment_lag_bucket (google.ads.googleads.v20.enums.types.ConversionOrAdjustmentLagBucketEnum.ConversionOrAdjustmentLagBucket):
            An enum value representing the number of days
            between the impression and the conversion or
            between the impression and adjustments to the
            conversion.
        date (str):
            Date to which metrics apply.
            yyyy-MM-dd format, for example, 2018-04-17.

            This field is a member of `oneof`_ ``_date``.
        day_of_week (google.ads.googleads.v20.enums.types.DayOfWeekEnum.DayOfWeek):
            Day of the week, for example, MONDAY.
        device (google.ads.googleads.v20.enums.types.DeviceEnum.Device):
            Device to which metrics apply.
        external_conversion_source (google.ads.googleads.v20.enums.types.ExternalConversionSourceEnum.ExternalConversionSource):
            External conversion source.
        geo_target_airport (str):
            Resource name of the geo target constant that
            represents an airport.

            This field is a member of `oneof`_ ``_geo_target_airport``.
        geo_target_canton (str):
            Resource name of the geo target constant that
            represents a canton.

            This field is a member of `oneof`_ ``_geo_target_canton``.
        geo_target_city (str):
            Resource name of the geo target constant that
            represents a city.

            This field is a member of `oneof`_ ``_geo_target_city``.
        geo_target_country (str):
            Resource name of the geo target constant that
            represents a country.

            This field is a member of `oneof`_ ``_geo_target_country``.
        geo_target_county (str):
            Resource name of the geo target constant that
            represents a county.

            This field is a member of `oneof`_ ``_geo_target_county``.
        geo_target_district (str):
            Resource name of the geo target constant that
            represents a district.

            This field is a member of `oneof`_ ``_geo_target_district``.
        geo_target_metro (str):
            Resource name of the geo target constant that
            represents a metro.

            This field is a member of `oneof`_ ``_geo_target_metro``.
        geo_target_most_specific_location (str):
            Resource name of the geo target constant that
            represents the most specific location.

            This field is a member of `oneof`_ ``_geo_target_most_specific_location``.
        geo_target_postal_code (str):
            Resource name of the geo target constant that
            represents a postal code.

            This field is a member of `oneof`_ ``_geo_target_postal_code``.
        geo_target_province (str):
            Resource name of the geo target constant that
            represents a province.

            This field is a member of `oneof`_ ``_geo_target_province``.
        geo_target_region (str):
            Resource name of the geo target constant that
            represents a region.

            This field is a member of `oneof`_ ``_geo_target_region``.
        geo_target_state (str):
            Resource name of the geo target constant that
            represents a state.

            This field is a member of `oneof`_ ``_geo_target_state``.
        hotel_booking_window_days (int):
            Hotel booking window in days.

            This field is a member of `oneof`_ ``_hotel_booking_window_days``.
        hotel_center_id (int):
            Hotel center ID.

            This field is a member of `oneof`_ ``_hotel_center_id``.
        hotel_check_in_date (str):
            Hotel check-in date. Formatted as yyyy-MM-dd.

            This field is a member of `oneof`_ ``_hotel_check_in_date``.
        hotel_check_in_day_of_week (google.ads.googleads.v20.enums.types.DayOfWeekEnum.DayOfWeek):
            Hotel check-in day of week.
        hotel_city (str):
            Hotel city.

            This field is a member of `oneof`_ ``_hotel_city``.
        hotel_class (int):
            Hotel class.

            This field is a member of `oneof`_ ``_hotel_class``.
        hotel_country (str):
            Hotel country.

            This field is a member of `oneof`_ ``_hotel_country``.
        hotel_date_selection_type (google.ads.googleads.v20.enums.types.HotelDateSelectionTypeEnum.HotelDateSelectionType):
            Hotel date selection type.
        hotel_length_of_stay (int):
            Hotel length of stay.

            This field is a member of `oneof`_ ``_hotel_length_of_stay``.
        hotel_rate_rule_id (str):
            Hotel rate rule ID.

            This field is a member of `oneof`_ ``_hotel_rate_rule_id``.
        hotel_rate_type (google.ads.googleads.v20.enums.types.HotelRateTypeEnum.HotelRateType):
            Hotel rate type.
        hotel_price_bucket (google.ads.googleads.v20.enums.types.HotelPriceBucketEnum.HotelPriceBucket):
            Hotel price bucket.
        hotel_state (str):
            Hotel state.

            This field is a member of `oneof`_ ``_hotel_state``.
        hour (int):
            Hour of day as a number between 0 and 23,
            inclusive.

            This field is a member of `oneof`_ ``_hour``.
        interaction_on_this_extension (bool):
            Only used with feed item metrics.
            Indicates whether the interaction metrics
            occurred on the feed item itself or a different
            extension or ad unit.

            This field is a member of `oneof`_ ``_interaction_on_this_extension``.
        keyword (google.ads.googleads.v20.common.types.Keyword):
            Keyword criterion.
        month (str):
            Month as represented by the date of the first
            day of a month. Formatted as yyyy-MM-dd.

            This field is a member of `oneof`_ ``_month``.
        month_of_year (google.ads.googleads.v20.enums.types.MonthOfYearEnum.MonthOfYear):
            Month of the year, for example, January.
        partner_hotel_id (str):
            Partner hotel ID.

            This field is a member of `oneof`_ ``_partner_hotel_id``.
        product_aggregator_id (int):
            Aggregator ID of the product.

            This field is a member of `oneof`_ ``_product_aggregator_id``.
        product_category_level1 (str):
            Category (level 1) of the product.

            This field is a member of `oneof`_ ``_product_category_level1``.
        product_category_level2 (str):
            Category (level 2) of the product.

            This field is a member of `oneof`_ ``_product_category_level2``.
        product_category_level3 (str):
            Category (level 3) of the product.

            This field is a member of `oneof`_ ``_product_category_level3``.
        product_category_level4 (str):
            Category (level 4) of the product.

            This field is a member of `oneof`_ ``_product_category_level4``.
        product_category_level5 (str):
            Category (level 5) of the product.

            This field is a member of `oneof`_ ``_product_category_level5``.
        product_brand (str):
            Brand of the product.

            This field is a member of `oneof`_ ``_product_brand``.
        product_channel (google.ads.googleads.v20.enums.types.ProductChannelEnum.ProductChannel):
            Channel of the product.
        product_channel_exclusivity (google.ads.googleads.v20.enums.types.ProductChannelExclusivityEnum.ProductChannelExclusivity):
            Channel exclusivity of the product.
        product_condition (google.ads.googleads.v20.enums.types.ProductConditionEnum.ProductCondition):
            Condition of the product.
        product_country (str):
            Resource name of the geo target constant for
            the country of sale of the product.

            This field is a member of `oneof`_ ``_product_country``.
        product_custom_attribute0 (str):
            Custom attribute 0 of the product.

            This field is a member of `oneof`_ ``_product_custom_attribute0``.
        product_custom_attribute1 (str):
            Custom attribute 1 of the product.

            This field is a member of `oneof`_ ``_product_custom_attribute1``.
        product_custom_attribute2 (str):
            Custom attribute 2 of the product.

            This field is a member of `oneof`_ ``_product_custom_attribute2``.
        product_custom_attribute3 (str):
            Custom attribute 3 of the product.

            This field is a member of `oneof`_ ``_product_custom_attribute3``.
        product_custom_attribute4 (str):
            Custom attribute 4 of the product.

            This field is a member of `oneof`_ ``_product_custom_attribute4``.
        product_feed_label (str):
            Feed label of the product.

            This field is a member of `oneof`_ ``_product_feed_label``.
        product_item_id (str):
            Item ID of the product.

            This field is a member of `oneof`_ ``_product_item_id``.
        product_language (str):
            Resource name of the language constant for
            the language of the product.

            This field is a member of `oneof`_ ``_product_language``.
        product_merchant_id (int):
            Merchant ID of the product.

            This field is a member of `oneof`_ ``_product_merchant_id``.
        product_store_id (str):
            Store ID of the product.

            This field is a member of `oneof`_ ``_product_store_id``.
        product_title (str):
            Title of the product.

            This field is a member of `oneof`_ ``_product_title``.
        product_type_l1 (str):
            Type (level 1) of the product.

            This field is a member of `oneof`_ ``_product_type_l1``.
        product_type_l2 (str):
            Type (level 2) of the product.

            This field is a member of `oneof`_ ``_product_type_l2``.
        product_type_l3 (str):
            Type (level 3) of the product.

            This field is a member of `oneof`_ ``_product_type_l3``.
        product_type_l4 (str):
            Type (level 4) of the product.

            This field is a member of `oneof`_ ``_product_type_l4``.
        product_type_l5 (str):
            Type (level 5) of the product.

            This field is a member of `oneof`_ ``_product_type_l5``.
        quarter (str):
            Quarter as represented by the date of the
            first day of a quarter. Uses the calendar year
            for quarters, for example, the second quarter of
            2018 starts on 2018-04-01. Formatted as
            yyyy-MM-dd.

            This field is a member of `oneof`_ ``_quarter``.
        travel_destination_city (str):
            The city the user is searching for at query
            time.

            This field is a member of `oneof`_ ``_travel_destination_city``.
        travel_destination_country (str):
            The country the user is searching for at
            query time.

            This field is a member of `oneof`_ ``_travel_destination_country``.
        travel_destination_region (str):
            The region the user is searching for at query
            time.

            This field is a member of `oneof`_ ``_travel_destination_region``.
        recommendation_type (google.ads.googleads.v20.enums.types.RecommendationTypeEnum.RecommendationType):
            Recommendation type.
        search_engine_results_page_type (google.ads.googleads.v20.enums.types.SearchEngineResultsPageTypeEnum.SearchEngineResultsPageType):
            Type of the search engine results page.
        search_subcategory (str):
            A search term subcategory. An empty string
            denotes the catch-all subcategory for search
            terms that didn't fit into another subcategory.

            This field is a member of `oneof`_ ``_search_subcategory``.
        search_term (str):
            A search term.

            This field is a member of `oneof`_ ``_search_term``.
        search_term_match_type (google.ads.googleads.v20.enums.types.SearchTermMatchTypeEnum.SearchTermMatchType):
            Match type of the keyword that triggered the
            ad, including variants.
        slot (google.ads.googleads.v20.enums.types.SlotEnum.Slot):
            Position of the ad.
        conversion_value_rule_primary_dimension (google.ads.googleads.v20.enums.types.ConversionValueRulePrimaryDimensionEnum.ConversionValueRulePrimaryDimension):
            Primary dimension of applied conversion value rules.
            NO_RULE_APPLIED shows the total recorded value of
            conversions that do not have a value rule applied. ORIGINAL
            shows the original value of conversions to which a value
            rule has been applied. GEO_LOCATION, DEVICE, AUDIENCE,
            ITINERARY show the net adjustment after value rules were
            applied.
        webpage (str):
            Resource name of the ad group criterion that
            represents webpage criterion.

            This field is a member of `oneof`_ ``_webpage``.
        week (str):
            Week as defined as Monday through Sunday, and
            represented by the date of Monday. Formatted as
            yyyy-MM-dd.

            This field is a member of `oneof`_ ``_week``.
        year (int):
            Year, formatted as yyyy.

            This field is a member of `oneof`_ ``_year``.
        sk_ad_network_fine_conversion_value (int):
            iOS Store Kit Ad Network conversion value.
            Null value means this segment is not applicable,
            for example, non-iOS campaign.

            This field is a member of `oneof`_ ``_sk_ad_network_fine_conversion_value``.
        sk_ad_network_redistributed_fine_conversion_value (int):
            iOS Store Kit Ad Network redistributed fine
            conversion value.
            Google uses modeling on observed conversion
            values(obtained from Apple) to calculate
            conversions from SKAN postbacks where NULLs are
            returned. This column represents the sum of the
            modeled conversion values and the observed
            conversion values. See
            https://support.google.com/google-ads/answer/14892597
            to lean more.

            This field is a member of `oneof`_ ``_sk_ad_network_redistributed_fine_conversion_value``.
        sk_ad_network_user_type (google.ads.googleads.v20.enums.types.SkAdNetworkUserTypeEnum.SkAdNetworkUserType):
            iOS Store Kit Ad Network user type.
        sk_ad_network_ad_event_type (google.ads.googleads.v20.enums.types.SkAdNetworkAdEventTypeEnum.SkAdNetworkAdEventType):
            iOS Store Kit Ad Network ad event type.
        sk_ad_network_source_app (google.ads.googleads.v20.common.types.SkAdNetworkSourceApp):
            App where the ad that drove the iOS Store Kit
            Ad Network install was shown. Null value means
            this segment is not applicable, for example,
            non-iOS campaign, or was not present in any
            postbacks sent by Apple.

            This field is a member of `oneof`_ ``_sk_ad_network_source_app``.
        sk_ad_network_attribution_credit (google.ads.googleads.v20.enums.types.SkAdNetworkAttributionCreditEnum.SkAdNetworkAttributionCredit):
            iOS Store Kit Ad Network attribution credit
        sk_ad_network_coarse_conversion_value (google.ads.googleads.v20.enums.types.SkAdNetworkCoarseConversionValueEnum.SkAdNetworkCoarseConversionValue):
            iOS Store Kit Ad Network coarse conversion
            value.
        sk_ad_network_source_domain (str):
            Website where the ad that drove the iOS Store
            Kit Ad Network install was shown. Null value
            means this segment is not applicable, for
            example, non-iOS campaign, or was not present in
            any postbacks sent by Apple.

            This field is a member of `oneof`_ ``_sk_ad_network_source_domain``.
        sk_ad_network_source_type (google.ads.googleads.v20.enums.types.SkAdNetworkSourceTypeEnum.SkAdNetworkSourceType):
            The source type where the ad that drove the
            iOS Store Kit Ad Network install was shown. Null
            value means this segment is not applicable, for
            example, non-iOS campaign, or neither source
            domain nor source app were present in any
            postbacks sent by Apple.
        sk_ad_network_postback_sequence_index (int):
            iOS Store Kit Ad Network postback sequence
            index.

            This field is a member of `oneof`_ ``_sk_ad_network_postback_sequence_index``.
        sk_ad_network_version (str):
            The version of the SKAdNetwork API used.

            This field is a member of `oneof`_ ``_sk_ad_network_version``.
        asset_interaction_target (google.ads.googleads.v20.common.types.AssetInteractionTarget):
            Only used with CustomerAsset, CampaignAsset and AdGroupAsset
            metrics. Indicates whether the interaction metrics occurred
            on the asset itself or a different asset or ad unit.
            Interactions (for example, clicks) are counted across all
            the parts of the served ad (for example, Ad itself and other
            components like Sitelinks) when they are served together.
            When interaction_on_this_asset is true, it means the
            interactions are on this specific asset and when
            interaction_on_this_asset is false, it means the
            interactions is not on this specific asset but on other
            parts of the served ad this asset is served with.

            This field is a member of `oneof`_ ``_asset_interaction_target``.
        new_versus_returning_customers (google.ads.googleads.v20.enums.types.ConvertingUserPriorEngagementTypeAndLtvBucketEnum.ConvertingUserPriorEngagementTypeAndLtvBucket):
            This is for segmenting conversions by whether
            the user is a new customer or a returning
            customer. This segmentation is typically used to
            measure the impact of customer acquisition goal.
        adjusted_age_range (google.ads.googleads.v20.enums.types.AgeRangeTypeEnum.AgeRangeType):
            Adjusted age range. This is the age range of the user after
            applying modeling to get more accurate age and gender
            information. Currently, both adjusted_age_range and
            adjusted_gender need to be selected together to get valid
            reach stats. These segmentations are only available for
            allowlisted customers.
        adjusted_gender (google.ads.googleads.v20.enums.types.GenderTypeEnum.GenderType):
            Adjusted gender. This is the gender of the user after
            applying modeling to get more accurate age and gender
            information. Currently, both adjusted_age_range and
            adjusted_gender need to be selected together to get valid
            reach stats. These segmentations are only available for
            allowlisted customers.
    """

    activity_account_id: int = proto.Field(
        proto.INT64,
        number=148,
        optional=True,
    )
    activity_city: str = proto.Field(
        proto.STRING,
        number=185,
        optional=True,
    )
    activity_country: str = proto.Field(
        proto.STRING,
        number=186,
        optional=True,
    )
    activity_rating: int = proto.Field(
        proto.INT64,
        number=149,
        optional=True,
    )
    activity_state: str = proto.Field(
        proto.STRING,
        number=187,
        optional=True,
    )
    external_activity_id: str = proto.Field(
        proto.STRING,
        number=150,
        optional=True,
    )
    ad_destination_type: (
        gage_ad_destination_type.AdDestinationTypeEnum.AdDestinationType
    ) = proto.Field(
        proto.ENUM,
        number=136,
        enum=gage_ad_destination_type.AdDestinationTypeEnum.AdDestinationType,
    )
    ad_format_type: gage_ad_format_type.AdFormatTypeEnum.AdFormatType = (
        proto.Field(
            proto.ENUM,
            number=191,
            enum=gage_ad_format_type.AdFormatTypeEnum.AdFormatType,
        )
    )
    ad_network_type: gage_ad_network_type.AdNetworkTypeEnum.AdNetworkType = (
        proto.Field(
            proto.ENUM,
            number=3,
            enum=gage_ad_network_type.AdNetworkTypeEnum.AdNetworkType,
        )
    )
    ad_group: str = proto.Field(
        proto.STRING,
        number=158,
        optional=True,
    )
    asset_group: str = proto.Field(
        proto.STRING,
        number=159,
        optional=True,
    )
    auction_insight_domain: str = proto.Field(
        proto.STRING,
        number=145,
        optional=True,
    )
    budget_campaign_association_status: "BudgetCampaignAssociationStatus" = (
        proto.Field(
            proto.MESSAGE,
            number=134,
            message="BudgetCampaignAssociationStatus",
        )
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=157,
        optional=True,
    )
    click_type: gage_click_type.ClickTypeEnum.ClickType = proto.Field(
        proto.ENUM,
        number=26,
        enum=gage_click_type.ClickTypeEnum.ClickType,
    )
    conversion_action: str = proto.Field(
        proto.STRING,
        number=113,
        optional=True,
    )
    conversion_action_category: (
        gage_conversion_action_category.ConversionActionCategoryEnum.ConversionActionCategory
    ) = proto.Field(
        proto.ENUM,
        number=53,
        enum=gage_conversion_action_category.ConversionActionCategoryEnum.ConversionActionCategory,
    )
    conversion_action_name: str = proto.Field(
        proto.STRING,
        number=114,
        optional=True,
    )
    conversion_adjustment: bool = proto.Field(
        proto.BOOL,
        number=115,
        optional=True,
    )
    conversion_attribution_event_type: (
        gage_conversion_attribution_event_type.ConversionAttributionEventTypeEnum.ConversionAttributionEventType
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=gage_conversion_attribution_event_type.ConversionAttributionEventTypeEnum.ConversionAttributionEventType,
    )
    conversion_lag_bucket: (
        gage_conversion_lag_bucket.ConversionLagBucketEnum.ConversionLagBucket
    ) = proto.Field(
        proto.ENUM,
        number=50,
        enum=gage_conversion_lag_bucket.ConversionLagBucketEnum.ConversionLagBucket,
    )
    conversion_or_adjustment_lag_bucket: (
        gage_conversion_or_adjustment_lag_bucket.ConversionOrAdjustmentLagBucketEnum.ConversionOrAdjustmentLagBucket
    ) = proto.Field(
        proto.ENUM,
        number=51,
        enum=gage_conversion_or_adjustment_lag_bucket.ConversionOrAdjustmentLagBucketEnum.ConversionOrAdjustmentLagBucket,
    )
    date: str = proto.Field(
        proto.STRING,
        number=79,
        optional=True,
    )
    day_of_week: gage_day_of_week.DayOfWeekEnum.DayOfWeek = proto.Field(
        proto.ENUM,
        number=5,
        enum=gage_day_of_week.DayOfWeekEnum.DayOfWeek,
    )
    device: gage_device.DeviceEnum.Device = proto.Field(
        proto.ENUM,
        number=1,
        enum=gage_device.DeviceEnum.Device,
    )
    external_conversion_source: (
        gage_external_conversion_source.ExternalConversionSourceEnum.ExternalConversionSource
    ) = proto.Field(
        proto.ENUM,
        number=55,
        enum=gage_external_conversion_source.ExternalConversionSourceEnum.ExternalConversionSource,
    )
    geo_target_airport: str = proto.Field(
        proto.STRING,
        number=116,
        optional=True,
    )
    geo_target_canton: str = proto.Field(
        proto.STRING,
        number=117,
        optional=True,
    )
    geo_target_city: str = proto.Field(
        proto.STRING,
        number=118,
        optional=True,
    )
    geo_target_country: str = proto.Field(
        proto.STRING,
        number=119,
        optional=True,
    )
    geo_target_county: str = proto.Field(
        proto.STRING,
        number=120,
        optional=True,
    )
    geo_target_district: str = proto.Field(
        proto.STRING,
        number=121,
        optional=True,
    )
    geo_target_metro: str = proto.Field(
        proto.STRING,
        number=122,
        optional=True,
    )
    geo_target_most_specific_location: str = proto.Field(
        proto.STRING,
        number=123,
        optional=True,
    )
    geo_target_postal_code: str = proto.Field(
        proto.STRING,
        number=124,
        optional=True,
    )
    geo_target_province: str = proto.Field(
        proto.STRING,
        number=125,
        optional=True,
    )
    geo_target_region: str = proto.Field(
        proto.STRING,
        number=126,
        optional=True,
    )
    geo_target_state: str = proto.Field(
        proto.STRING,
        number=127,
        optional=True,
    )
    hotel_booking_window_days: int = proto.Field(
        proto.INT64,
        number=135,
        optional=True,
    )
    hotel_center_id: int = proto.Field(
        proto.INT64,
        number=80,
        optional=True,
    )
    hotel_check_in_date: str = proto.Field(
        proto.STRING,
        number=81,
        optional=True,
    )
    hotel_check_in_day_of_week: gage_day_of_week.DayOfWeekEnum.DayOfWeek = (
        proto.Field(
            proto.ENUM,
            number=9,
            enum=gage_day_of_week.DayOfWeekEnum.DayOfWeek,
        )
    )
    hotel_city: str = proto.Field(
        proto.STRING,
        number=82,
        optional=True,
    )
    hotel_class: int = proto.Field(
        proto.INT32,
        number=83,
        optional=True,
    )
    hotel_country: str = proto.Field(
        proto.STRING,
        number=84,
        optional=True,
    )
    hotel_date_selection_type: (
        gage_hotel_date_selection_type.HotelDateSelectionTypeEnum.HotelDateSelectionType
    ) = proto.Field(
        proto.ENUM,
        number=13,
        enum=gage_hotel_date_selection_type.HotelDateSelectionTypeEnum.HotelDateSelectionType,
    )
    hotel_length_of_stay: int = proto.Field(
        proto.INT32,
        number=85,
        optional=True,
    )
    hotel_rate_rule_id: str = proto.Field(
        proto.STRING,
        number=86,
        optional=True,
    )
    hotel_rate_type: gage_hotel_rate_type.HotelRateTypeEnum.HotelRateType = (
        proto.Field(
            proto.ENUM,
            number=74,
            enum=gage_hotel_rate_type.HotelRateTypeEnum.HotelRateType,
        )
    )
    hotel_price_bucket: (
        gage_hotel_price_bucket.HotelPriceBucketEnum.HotelPriceBucket
    ) = proto.Field(
        proto.ENUM,
        number=78,
        enum=gage_hotel_price_bucket.HotelPriceBucketEnum.HotelPriceBucket,
    )
    hotel_state: str = proto.Field(
        proto.STRING,
        number=87,
        optional=True,
    )
    hour: int = proto.Field(
        proto.INT32,
        number=88,
        optional=True,
    )
    interaction_on_this_extension: bool = proto.Field(
        proto.BOOL,
        number=89,
        optional=True,
    )
    keyword: "Keyword" = proto.Field(
        proto.MESSAGE,
        number=61,
        message="Keyword",
    )
    month: str = proto.Field(
        proto.STRING,
        number=90,
        optional=True,
    )
    month_of_year: gage_month_of_year.MonthOfYearEnum.MonthOfYear = proto.Field(
        proto.ENUM,
        number=18,
        enum=gage_month_of_year.MonthOfYearEnum.MonthOfYear,
    )
    partner_hotel_id: str = proto.Field(
        proto.STRING,
        number=91,
        optional=True,
    )
    product_aggregator_id: int = proto.Field(
        proto.INT64,
        number=132,
        optional=True,
    )
    product_category_level1: str = proto.Field(
        proto.STRING,
        number=161,
        optional=True,
    )
    product_category_level2: str = proto.Field(
        proto.STRING,
        number=162,
        optional=True,
    )
    product_category_level3: str = proto.Field(
        proto.STRING,
        number=163,
        optional=True,
    )
    product_category_level4: str = proto.Field(
        proto.STRING,
        number=164,
        optional=True,
    )
    product_category_level5: str = proto.Field(
        proto.STRING,
        number=165,
        optional=True,
    )
    product_brand: str = proto.Field(
        proto.STRING,
        number=97,
        optional=True,
    )
    product_channel: gage_product_channel.ProductChannelEnum.ProductChannel = (
        proto.Field(
            proto.ENUM,
            number=30,
            enum=gage_product_channel.ProductChannelEnum.ProductChannel,
        )
    )
    product_channel_exclusivity: (
        gage_product_channel_exclusivity.ProductChannelExclusivityEnum.ProductChannelExclusivity
    ) = proto.Field(
        proto.ENUM,
        number=31,
        enum=gage_product_channel_exclusivity.ProductChannelExclusivityEnum.ProductChannelExclusivity,
    )
    product_condition: (
        gage_product_condition.ProductConditionEnum.ProductCondition
    ) = proto.Field(
        proto.ENUM,
        number=32,
        enum=gage_product_condition.ProductConditionEnum.ProductCondition,
    )
    product_country: str = proto.Field(
        proto.STRING,
        number=98,
        optional=True,
    )
    product_custom_attribute0: str = proto.Field(
        proto.STRING,
        number=99,
        optional=True,
    )
    product_custom_attribute1: str = proto.Field(
        proto.STRING,
        number=100,
        optional=True,
    )
    product_custom_attribute2: str = proto.Field(
        proto.STRING,
        number=101,
        optional=True,
    )
    product_custom_attribute3: str = proto.Field(
        proto.STRING,
        number=102,
        optional=True,
    )
    product_custom_attribute4: str = proto.Field(
        proto.STRING,
        number=103,
        optional=True,
    )
    product_feed_label: str = proto.Field(
        proto.STRING,
        number=147,
        optional=True,
    )
    product_item_id: str = proto.Field(
        proto.STRING,
        number=104,
        optional=True,
    )
    product_language: str = proto.Field(
        proto.STRING,
        number=105,
        optional=True,
    )
    product_merchant_id: int = proto.Field(
        proto.INT64,
        number=133,
        optional=True,
    )
    product_store_id: str = proto.Field(
        proto.STRING,
        number=106,
        optional=True,
    )
    product_title: str = proto.Field(
        proto.STRING,
        number=107,
        optional=True,
    )
    product_type_l1: str = proto.Field(
        proto.STRING,
        number=108,
        optional=True,
    )
    product_type_l2: str = proto.Field(
        proto.STRING,
        number=109,
        optional=True,
    )
    product_type_l3: str = proto.Field(
        proto.STRING,
        number=110,
        optional=True,
    )
    product_type_l4: str = proto.Field(
        proto.STRING,
        number=111,
        optional=True,
    )
    product_type_l5: str = proto.Field(
        proto.STRING,
        number=112,
        optional=True,
    )
    quarter: str = proto.Field(
        proto.STRING,
        number=128,
        optional=True,
    )
    travel_destination_city: str = proto.Field(
        proto.STRING,
        number=193,
        optional=True,
    )
    travel_destination_country: str = proto.Field(
        proto.STRING,
        number=194,
        optional=True,
    )
    travel_destination_region: str = proto.Field(
        proto.STRING,
        number=195,
        optional=True,
    )
    recommendation_type: (
        gage_recommendation_type.RecommendationTypeEnum.RecommendationType
    ) = proto.Field(
        proto.ENUM,
        number=140,
        enum=gage_recommendation_type.RecommendationTypeEnum.RecommendationType,
    )
    search_engine_results_page_type: (
        gage_search_engine_results_page_type.SearchEngineResultsPageTypeEnum.SearchEngineResultsPageType
    ) = proto.Field(
        proto.ENUM,
        number=70,
        enum=gage_search_engine_results_page_type.SearchEngineResultsPageTypeEnum.SearchEngineResultsPageType,
    )
    search_subcategory: str = proto.Field(
        proto.STRING,
        number=155,
        optional=True,
    )
    search_term: str = proto.Field(
        proto.STRING,
        number=156,
        optional=True,
    )
    search_term_match_type: (
        gage_search_term_match_type.SearchTermMatchTypeEnum.SearchTermMatchType
    ) = proto.Field(
        proto.ENUM,
        number=22,
        enum=gage_search_term_match_type.SearchTermMatchTypeEnum.SearchTermMatchType,
    )
    slot: gage_slot.SlotEnum.Slot = proto.Field(
        proto.ENUM,
        number=23,
        enum=gage_slot.SlotEnum.Slot,
    )
    conversion_value_rule_primary_dimension: (
        gage_conversion_value_rule_primary_dimension.ConversionValueRulePrimaryDimensionEnum.ConversionValueRulePrimaryDimension
    ) = proto.Field(
        proto.ENUM,
        number=138,
        enum=gage_conversion_value_rule_primary_dimension.ConversionValueRulePrimaryDimensionEnum.ConversionValueRulePrimaryDimension,
    )
    webpage: str = proto.Field(
        proto.STRING,
        number=129,
        optional=True,
    )
    week: str = proto.Field(
        proto.STRING,
        number=130,
        optional=True,
    )
    year: int = proto.Field(
        proto.INT32,
        number=131,
        optional=True,
    )
    sk_ad_network_fine_conversion_value: int = proto.Field(
        proto.INT64,
        number=137,
        optional=True,
    )
    sk_ad_network_redistributed_fine_conversion_value: int = proto.Field(
        proto.INT64,
        number=190,
        optional=True,
    )
    sk_ad_network_user_type: (
        gage_sk_ad_network_user_type.SkAdNetworkUserTypeEnum.SkAdNetworkUserType
    ) = proto.Field(
        proto.ENUM,
        number=141,
        enum=gage_sk_ad_network_user_type.SkAdNetworkUserTypeEnum.SkAdNetworkUserType,
    )
    sk_ad_network_ad_event_type: (
        gage_sk_ad_network_ad_event_type.SkAdNetworkAdEventTypeEnum.SkAdNetworkAdEventType
    ) = proto.Field(
        proto.ENUM,
        number=142,
        enum=gage_sk_ad_network_ad_event_type.SkAdNetworkAdEventTypeEnum.SkAdNetworkAdEventType,
    )
    sk_ad_network_source_app: "SkAdNetworkSourceApp" = proto.Field(
        proto.MESSAGE,
        number=143,
        optional=True,
        message="SkAdNetworkSourceApp",
    )
    sk_ad_network_attribution_credit: (
        gage_sk_ad_network_attribution_credit.SkAdNetworkAttributionCreditEnum.SkAdNetworkAttributionCredit
    ) = proto.Field(
        proto.ENUM,
        number=144,
        enum=gage_sk_ad_network_attribution_credit.SkAdNetworkAttributionCreditEnum.SkAdNetworkAttributionCredit,
    )
    sk_ad_network_coarse_conversion_value: (
        gage_sk_ad_network_coarse_conversion_value.SkAdNetworkCoarseConversionValueEnum.SkAdNetworkCoarseConversionValue
    ) = proto.Field(
        proto.ENUM,
        number=151,
        enum=gage_sk_ad_network_coarse_conversion_value.SkAdNetworkCoarseConversionValueEnum.SkAdNetworkCoarseConversionValue,
    )
    sk_ad_network_source_domain: str = proto.Field(
        proto.STRING,
        number=152,
        optional=True,
    )
    sk_ad_network_source_type: (
        gage_sk_ad_network_source_type.SkAdNetworkSourceTypeEnum.SkAdNetworkSourceType
    ) = proto.Field(
        proto.ENUM,
        number=153,
        enum=gage_sk_ad_network_source_type.SkAdNetworkSourceTypeEnum.SkAdNetworkSourceType,
    )
    sk_ad_network_postback_sequence_index: int = proto.Field(
        proto.INT64,
        number=154,
        optional=True,
    )
    sk_ad_network_version: str = proto.Field(
        proto.STRING,
        number=192,
        optional=True,
    )
    asset_interaction_target: "AssetInteractionTarget" = proto.Field(
        proto.MESSAGE,
        number=139,
        optional=True,
        message="AssetInteractionTarget",
    )
    new_versus_returning_customers: (
        converting_user_prior_engagement_type_and_ltv_bucket.ConvertingUserPriorEngagementTypeAndLtvBucketEnum.ConvertingUserPriorEngagementTypeAndLtvBucket
    ) = proto.Field(
        proto.ENUM,
        number=160,
        enum=converting_user_prior_engagement_type_and_ltv_bucket.ConvertingUserPriorEngagementTypeAndLtvBucketEnum.ConvertingUserPriorEngagementTypeAndLtvBucket,
    )
    adjusted_age_range: age_range_type.AgeRangeTypeEnum.AgeRangeType = (
        proto.Field(
            proto.ENUM,
            number=196,
            enum=age_range_type.AgeRangeTypeEnum.AgeRangeType,
        )
    )
    adjusted_gender: gender_type.GenderTypeEnum.GenderType = proto.Field(
        proto.ENUM,
        number=197,
        enum=gender_type.GenderTypeEnum.GenderType,
    )


class Keyword(proto.Message):
    r"""A Keyword criterion segment.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        ad_group_criterion (str):
            The AdGroupCriterion resource name.

            This field is a member of `oneof`_ ``_ad_group_criterion``.
        info (google.ads.googleads.v20.common.types.KeywordInfo):
            Keyword info.
    """

    ad_group_criterion: str = proto.Field(
        proto.STRING,
        number=3,
        optional=True,
    )
    info: criteria.KeywordInfo = proto.Field(
        proto.MESSAGE,
        number=2,
        message=criteria.KeywordInfo,
    )


class BudgetCampaignAssociationStatus(proto.Message):
    r"""A BudgetCampaignAssociationStatus segment.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        campaign (str):
            The campaign resource name.

            This field is a member of `oneof`_ ``_campaign``.
        status (google.ads.googleads.v20.enums.types.BudgetCampaignAssociationStatusEnum.BudgetCampaignAssociationStatus):
            Budget campaign association status.
    """

    campaign: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )
    status: (
        gage_budget_campaign_association_status.BudgetCampaignAssociationStatusEnum.BudgetCampaignAssociationStatus
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=gage_budget_campaign_association_status.BudgetCampaignAssociationStatusEnum.BudgetCampaignAssociationStatus,
    )


class AssetInteractionTarget(proto.Message):
    r"""An AssetInteractionTarget segment.

    Attributes:
        asset (str):
            The asset resource name.
        interaction_on_this_asset (bool):
            Only used with CustomerAsset, CampaignAsset
            and AdGroupAsset metrics. Indicates whether the
            interaction metrics occurred on the asset itself
            or a different asset or ad unit.
    """

    asset: str = proto.Field(
        proto.STRING,
        number=1,
    )
    interaction_on_this_asset: bool = proto.Field(
        proto.BOOL,
        number=2,
    )


class SkAdNetworkSourceApp(proto.Message):
    r"""A SkAdNetworkSourceApp segment.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        sk_ad_network_source_app_id (str):
            App id where the ad that drove the iOS Store
            Kit Ad Network install was shown.

            This field is a member of `oneof`_ ``_sk_ad_network_source_app_id``.
    """

    sk_ad_network_source_app_id: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.common.types import criteria
from google.ads.googleads.v22.enums.types import campaign_criterion_status
from google.ads.googleads.v22.enums.types import criterion_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "CampaignCriterion",
    },
)


class CampaignCriterion(proto.Message):
    r"""A campaign criterion.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the campaign criterion.
            Campaign criterion resource names have the form:

            ``customers/{customer_id}/campaignCriteria/{campaign_id}~{criterion_id}``
        campaign (str):
            Immutable. The campaign to which the
            criterion belongs.

            This field is a member of `oneof`_ ``_campaign``.
        criterion_id (int):
            Output only. The ID of the criterion.

            This field is ignored during mutate.

            This field is a member of `oneof`_ ``_criterion_id``.
        display_name (str):
            Output only. The display name of the
            criterion.
            This field is ignored for mutates.
        bid_modifier (float):
            The modifier for the bids when the criterion
            matches. The modifier must be in the range: 0.1
            - 10.0. Most targetable criteria types support
            modifiers. Use 0 to opt out of a Device type.

            This field is a member of `oneof`_ ``_bid_modifier``.
        negative (bool):
            Immutable. Whether to target (``false``) or exclude
            (``true``) the criterion.

            This field is a member of `oneof`_ ``_negative``.
        type_ (google.ads.googleads.v22.enums.types.CriterionTypeEnum.CriterionType):
            Output only. The type of the criterion.
        status (google.ads.googleads.v22.enums.types.CampaignCriterionStatusEnum.CampaignCriterionStatus):
            The status of the criterion.
        keyword (google.ads.googleads.v22.common.types.KeywordInfo):
            Immutable. Keyword.

            This field is a member of `oneof`_ ``criterion``.
        placement (google.ads.googleads.v22.common.types.PlacementInfo):
            Immutable. Placement.

            This field is a member of `oneof`_ ``criterion``.
        mobile_app_category (google.ads.googleads.v22.common.types.MobileAppCategoryInfo):
            Immutable. Mobile app category.

            This field is a member of `oneof`_ ``criterion``.
        mobile_application (google.ads.googleads.v22.common.types.MobileApplicationInfo):
            Immutable. Mobile application.

            This field is a member of `oneof`_ ``criterion``.
        location (google.ads.googleads.v22.common.types.LocationInfo):
            Immutable. Location.

            This field is a member of `oneof`_ ``criterion``.
        device (google.ads.googleads.v22.common.types.DeviceInfo):
            Immutable. Device.

            This field is a member of `oneof`_ ``criterion``.
        ad_schedule (google.ads.googleads.v22.common.types.AdScheduleInfo):
            Immutable. Ad Schedule.

            This field is a member of `oneof`_ ``criterion``.
        age_range (google.ads.googleads.v22.common.types.AgeRangeInfo):
            Immutable. Age range.

            This field is a member of `oneof`_ ``criterion``.
        gender (google.ads.googleads.v22.common.types.GenderInfo):
            Immutable. Gender.

            This field is a member of `oneof`_ ``criterion``.
        income_range (google.ads.googleads.v22.common.types.IncomeRangeInfo):
            Immutable. Income range.

            This field is a member of `oneof`_ ``criterion``.
        parental_status (google.ads.googleads.v22.common.types.ParentalStatusInfo):
            Immutable. Parental status.

            This field is a member of `oneof`_ ``criterion``.
        user_list (google.ads.googleads.v22.common.types.UserListInfo):
            Immutable. User List.

            This field is a member of `oneof`_ ``criterion``.
        youtube_video (google.ads.googleads.v22.common.types.YouTubeVideoInfo):
            Immutable. YouTube Video.

            This field is a member of `oneof`_ ``criterion``.
        youtube_channel (google.ads.googleads.v22.common.types.YouTubeChannelInfo):
            Immutable. YouTube Channel.

            This field is a member of `oneof`_ ``criterion``.
        proximity (google.ads.googleads.v22.common.types.ProximityInfo):
            Immutable. Proximity.

            This field is a member of `oneof`_ ``criterion``.
        topic (google.ads.googleads.v22.common.types.TopicInfo):
            Immutable. Topic.

            This field is a member of `oneof`_ ``criterion``.
        listing_scope (google.ads.googleads.v22.common.types.ListingScopeInfo):
            Immutable. Listing scope.

            This field is a member of `oneof`_ ``criterion``.
        language (google.ads.googleads.v22.common.types.LanguageInfo):
            Immutable. Language.

            This field is a member of `oneof`_ ``criterion``.
        ip_block (google.ads.googleads.v22.common.types.IpBlockInfo):
            Immutable. IpBlock.

            This field is a member of `oneof`_ ``criterion``.
        content_label (google.ads.googleads.v22.common.types.ContentLabelInfo):
            Immutable. ContentLabel.

            This field is a member of `oneof`_ ``criterion``.
        carrier (google.ads.googleads.v22.common.types.CarrierInfo):
            Immutable. Carrier.

            This field is a member of `oneof`_ ``criterion``.
        user_interest (google.ads.googleads.v22.common.types.UserInterestInfo):
            Immutable. User Interest.

            This field is a member of `oneof`_ ``criterion``.
        webpage (google.ads.googleads.v22.common.types.WebpageInfo):
            Immutable. Webpage.

            This field is a member of `oneof`_ ``criterion``.
        operating_system_version (google.ads.googleads.v22.common.types.OperatingSystemVersionInfo):
            Immutable. Operating system version.

            This field is a member of `oneof`_ ``criterion``.
        mobile_device (google.ads.googleads.v22.common.types.MobileDeviceInfo):
            Immutable. Mobile Device.

            This field is a member of `oneof`_ ``criterion``.
        location_group (google.ads.googleads.v22.common.types.LocationGroupInfo):
            Immutable. Location Group

            This field is a member of `oneof`_ ``criterion``.
        custom_affinity (google.ads.googleads.v22.common.types.CustomAffinityInfo):
            Immutable. Custom Affinity.

            This field is a member of `oneof`_ ``criterion``.
        custom_audience (google.ads.googleads.v22.common.types.CustomAudienceInfo):
            Immutable. Custom Audience

            This field is a member of `oneof`_ ``criterion``.
        combined_audience (google.ads.googleads.v22.common.types.CombinedAudienceInfo):
            Immutable. Combined Audience.

            This field is a member of `oneof`_ ``criterion``.
        keyword_theme (google.ads.googleads.v22.common.types.KeywordThemeInfo):
            Immutable. Smart Campaign Keyword Theme.

            This field is a member of `oneof`_ ``criterion``.
        local_service_id (google.ads.googleads.v22.common.types.LocalServiceIdInfo):
            Immutable. GLS service campaign criterion.

            This field is a member of `oneof`_ ``criterion``.
        brand_list (google.ads.googleads.v22.common.types.BrandListInfo):
            Immutable. Brand list campaign criterion.

            This field is a member of `oneof`_ ``criterion``.
        life_event (google.ads.googleads.v22.common.types.LifeEventInfo):
            Immutable. Life event campaign criterion.

            This field is a member of `oneof`_ ``criterion``.
        webpage_list (google.ads.googleads.v22.common.types.WebpageListInfo):
            Immutable. Webpage list.
            This criterion is not publicly available.

            This field is a member of `oneof`_ ``criterion``.
        video_lineup (google.ads.googleads.v22.common.types.VideoLineupInfo):
            Immutable. Video lineup criterion.

            This field is a member of `oneof`_ ``criterion``.
        extended_demographic (google.ads.googleads.v22.common.types.ExtendedDemographicInfo):
            Immutable. Extended demographic criterion.

            This field is a member of `oneof`_ ``criterion``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=37,
        optional=True,
    )
    criterion_id: int = proto.Field(
        proto.INT64,
        number=38,
        optional=True,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=43,
    )
    bid_modifier: float = proto.Field(
        proto.FLOAT,
        number=39,
        optional=True,
    )
    negative: bool = proto.Field(
        proto.BOOL,
        number=40,
        optional=True,
    )
    type_: criterion_type.CriterionTypeEnum.CriterionType = proto.Field(
        proto.ENUM,
        number=6,
        enum=criterion_type.CriterionTypeEnum.CriterionType,
    )
    status: (
        campaign_criterion_status.CampaignCriterionStatusEnum.CampaignCriterionStatus
    ) = proto.Field(
        proto.ENUM,
        number=35,
        enum=campaign_criterion_status.CampaignCriterionStatusEnum.CampaignCriterionStatus,
    )
    keyword: criteria.KeywordInfo = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="criterion",
        message=criteria.KeywordInfo,
    )
    placement: criteria.PlacementInfo = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="criterion",
        message=criteria.PlacementInfo,
    )
    mobile_app_category: criteria.MobileAppCategoryInfo = proto.Field(
        proto.MESSAGE,
        number=10,
        oneof="criterion",
        message=criteria.MobileAppCategoryInfo,
    )
    mobile_application: criteria.MobileApplicationInfo = proto.Field(
        proto.MESSAGE,
        number=11,
        oneof="criterion",
        message=criteria.MobileApplicationInfo,
    )
    location: criteria.LocationInfo = proto.Field(
        proto.MESSAGE,
        number=12,
        oneof="criterion",
        message=criteria.LocationInfo,
    )
    device: criteria.DeviceInfo = proto.Field(
        proto.MESSAGE,
        number=13,
        oneof="criterion",
        message=criteria.DeviceInfo,
    )
    ad_schedule: criteria.AdScheduleInfo = proto.Field(
        proto.MESSAGE,
        number=15,
        oneof="criterion",
        message=criteria.AdScheduleInfo,
    )
    age_range: criteria.AgeRangeInfo = proto.Field(
        proto.MESSAGE,
        number=16,
        oneof="criterion",
        message=criteria.AgeRangeInfo,
    )
    gender: criteria.GenderInfo = proto.Field(
        proto.MESSAGE,
        number=17,
        oneof="criterion",
        message=criteria.GenderInfo,
    )
    income_range: criteria.IncomeRangeInfo = proto.Field(
        proto.MESSAGE,
        number=18,
        oneof="criterion",
        message=criteria.IncomeRangeInfo,
    )
    parental_status: criteria.ParentalStatusInfo = proto.Field(
        proto.MESSAGE,
        number=19,
        oneof="criterion",
        message=criteria.ParentalStatusInfo,
    )
    user_list: criteria.UserListInfo = proto.Field(
        proto.MESSAGE,
        number=22,
        oneof="criterion",
        message=criteria.UserListInfo,
    )
    youtube_video: criteria.YouTubeVideoInfo = proto.Field(
        proto.MESSAGE,
        number=20,
        oneof="criterion",
        message=criteria.YouTubeVideoInfo,
    )
    youtube_channel: criteria.YouTubeChannelInfo = proto.Field(
        proto.MESSAGE,
        number=21,
        oneof="criterion",
        message=criteria.YouTubeChannelInfo,
    )
    proximity: criteria.ProximityInfo = proto.Field(
        proto.MESSAGE,
        number=23,
        oneof="criterion",
        message=criteria.ProximityInfo,
    )
    topic: criteria.TopicInfo = proto.Field(
        proto.MESSAGE,
        number=24,
        oneof="criterion",
        message=criteria.TopicInfo,
    )
    listing_scope: criteria.ListingScopeInfo = proto.Field(
        proto.MESSAGE,
        number=25,
        oneof="criterion",
        message=criteria.ListingScopeInfo,
    )
    language: criteria.LanguageInfo = proto.Field(
        proto.MESSAGE,
        number=26,
        oneof="criterion",
        message=criteria.LanguageInfo,
    )
    ip_block: criteria.IpBlockInfo = proto.Field(
        proto.MESSAGE,
        number=27,
        oneof="criterion",
        message=criteria.IpBlockInfo,
    )
    content_label: criteria.ContentLabelInfo = proto.Field(
        proto.MESSAGE,
        number=28,
        oneof="criterion",
        message=criteria.ContentLabelInfo,
    )
    carrier: criteria.CarrierInfo = proto.Field(
        proto.MESSAGE,
        number=29,
        oneof="criterion",
        message=criteria.CarrierInfo,
    )
    user_interest: criteria.UserInterestInfo = proto.Field(
        proto.MESSAGE,
        number=30,
        oneof="criterion",
        message=criteria.UserInterestInfo,
    )
    webpage: criteria.WebpageInfo = proto.Field(
        proto.MESSAGE,
        number=31,
        oneof="criterion",
        message=criteria.WebpageInfo,
    )
    operating_system_version: criteria.OperatingSystemVersionInfo = proto.Field(
        proto.MESSAGE,
        number=32,
        oneof="criterion",
        message=criteria.OperatingSystemVersionInfo,
    )
    mobile_device: criteria.MobileDeviceInfo = proto.Field(
        proto.MESSAGE,
        number=33,
        oneof="criterion",
        message=criteria.MobileDeviceInfo,
    )
    location_group: criteria.LocationGroupInfo = proto.Field(
        proto.MESSAGE,
        number=34,
        oneof="criterion",
        message=criteria.LocationGroupInfo,
    )
    custom_affinity: criteria.CustomAffinityInfo = proto.Field(
        proto.MESSAGE,
        number=36,
        oneof="criterion",
        message=criteria.CustomAffinityInfo,
    )
    custom_audience: criteria.CustomAudienceInfo = proto.Field(
        proto.MESSAGE,
        number=41,
        oneof="criterion",
        message=criteria.CustomAudienceInfo,
    )
    combined_audience: criteria.CombinedAudienceInfo = proto.Field(
        proto.MESSAGE,
        number=42,
        oneof="criterion",
        message=criteria.CombinedAudienceInfo,
    )
    keyword_theme: criteria.KeywordThemeInfo = proto.Field(
        proto.MESSAGE,
        number=45,
        oneof="criterion",
        message=criteria.KeywordThemeInfo,
    )
    local_service_id: criteria.LocalServiceIdInfo = proto.Field(
        proto.MESSAGE,
        number=46,
        oneof="criterion",
        message=criteria.LocalServiceIdInfo,
    )
    brand_list: criteria.BrandListInfo = proto.Field(
        proto.MESSAGE,
        number=47,
        oneof="criterion",
        message=criteria.BrandListInfo,
    )
    life_event: criteria.LifeEventInfo = proto.Field(
        proto.MESSAGE,
        number=48,
        oneof="criterion",
        message=criteria.LifeEventInfo,
    )
    webpage_list: criteria.WebpageListInfo = proto.Field(
        proto.MESSAGE,
        number=49,
        oneof="criterion",
        message=criteria.WebpageListInfo,
    )
    video_lineup: criteria.VideoLineupInfo = proto.Field(
        proto.MESSAGE,
        number=50,
        oneof="criterion",
        message=criteria.VideoLineupInfo,
    )
    extended_demographic: criteria.ExtendedDemographicInfo = proto.Field(
        proto.MESSAGE,
        number=52,
        oneof="criterion",
        message=criteria.ExtendedDemographicInfo,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Adds a display upload ad to a given ad group.

To get ad groups, run get_ad_groups.py.
"""


import argparse
import sys

import requests

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.resources.types.ad import Ad
from google.ads.googleads.v22.resources.types.ad_group_ad import AdGroupAd
from google.ads.googleads.v22.resources.types.asset import Asset
from google.ads.googleads.v22.services.services.ad_group_ad_service import (
    AdGroupAdServiceClient,
)
from google.ads.googleads.v22.services.services.asset_service import (
    AssetServiceClient,
)
from google.ads.googleads.v22.services.types.ad_group_ad_service import (
    AdGroupAdOperation,
    MutateAdGroupAdsResponse,
)
from google.ads.googleads.v22.services.types.asset_service import (
    AssetOperation,
    MutateAssetsResponse,
)


BUNDLE_URL: str = "https://gaagl.page.link/ib87"


def main(client: GoogleAdsClient, customer_id: str, ad_group_id: str) -> None:
    """Adds a display upload ad to a given ad group.

    Args:
        client: An initialized Google Ads client.
        customer_id: The Google Ads customer ID.
        ad_group_id: The ID of the ad group to which the new ad will be added.
    """
    # There are several types of display upload ads. For this example, we will
    # create an HTML5 upload ad, which requires a media bundle.
    # This feature is only available to allowlisted accounts.
    # See https://support.google.com/google-ads/answer/1722096 for more details.
    # The DisplayUploadProductType field lists the available display upload types:
    # https://developers.google.com/google-ads/api/reference/rpc/latest/DisplayUploadAdInfo

    # Creates a new media bundle asset and returns the resource name.
    ad_asset_resource_name: str = create_media_bundle_asset(client, customer_id)

    # Creates a new display upload ad and associates it with the specified
    # ad group.
    create_display_upload_ad_group_ad(
        client, customer_id, ad_group_id, ad_asset_resource_name
    )


def create_media_bundle_asset(client: GoogleAdsClient, customer_id: str) -> str:
    """Creates a media bundle from the assets in a zip file.

    The zip file contains the HTML5 components.

    Args:
        client: An initialized Google Ads client.
        customer_id: The Google Ads customer ID for which the call is made.
    Returns:
        The string resource name of the newly uploaded media bundle.
    """
    # Get the AssetService client.
    asset_service: AssetServiceClient = client.get_service("AssetService")

    # Construct an asset operation and populate its fields.
    asset_operation: AssetOperation = client.get_type("AssetOperation")
    media_bundle_asset: Asset = asset_operation.create
    media_bundle_asset.type_ = client.enums.AssetTypeEnum.MEDIA_BUNDLE
    media_bundle_asset.name = "Ad Media Bundle"
    # The HTML5 zip file contains all the HTML, CSS, and images needed for the
    # HTML5 ad. For help on creating an HTML5 zip file, check out Google Web
    # Designer (https://www.google.com/webdesigner/).
    # Download the ZIP as bytes from the URL
    media_bundle_asset.media_bundle_asset.data = requests.get(
        BUNDLE_URL
    ).content

    # Adds the asset to the client account.
    mutate_asset_response: MutateAssetsResponse = asset_service.mutate_assets(
        customer_id=customer_id, operations=[asset_operation]
    )

    # Display and return the resulting resource name.
    uploaded_asset_resource_name: str = mutate_asset_response.results[
        0
    ].resource_name
    print(f"Uploaded file with resource name '{uploaded_asset_resource_name}'.")

    return uploaded_asset_resource_name


def create_display_upload_ad_group_ad(
    client: GoogleAdsClient,
    customer_id: str,
    ad_group_id: str,
    ad_asset_resource_name: str,
) -> None:
    """Creates a new HTML5 display upload ad and adds it to the given ad group.

    Args:
        client: An initialized Google Ads client.
        customer_id: The Google Ads customer ID.
        ad_group_id: The ID of the ad group to which the new ad will be added.
        ad_asset_resource_name: The resource name of the media bundle containing
            the HTML5 components.
    """
    # Get the AdGroupAdService client.
    ad_group_ad_service: AdGroupAdServiceClient = client.get_service(
        "AdGroupAdService"
    )

    # Create an AdGroupAdOperation.
    ad_group_ad_operation: AdGroupAdOperation = client.get_type(
        "AdGroupAdOperation"
    )

    # Configure the ad group ad fields.
    ad_group_ad: AdGroupAd = ad_group_ad_operation.create
    ad_group_ad.status = client.enums.AdGroupAdStatusEnum.PAUSED
    ad_group_ad.ad_group = client.get_service("AdGroupService").ad_group_path(
        customer_id, ad_group_id
    )

    # Configured the ad as a display upload ad.
    display_upload_ad: Ad = ad_group_ad.ad
    display_upload_ad.name = "Ad for HTML5"
    display_upload_ad.final_urls.append("http://example.com/html5")
    # Exactly one of the ad_data "oneof" fields must be included to specify the
    # ad type. See: https://developers.google.com/google-ads/api/reference/rpc/latest/Ad for
    # the full list of available types. By setting a "display_upload_ad"
    # subfield it sets that as the "oneof" field for the Ad.
    display_upload_ad.display_upload_ad.media_bundle.asset = (
        ad_asset_resource_name
    )
    display_upload_ad.display_upload_ad.display_upload_product_type = (
        client.enums.DisplayUploadProductTypeEnum.HTML5_UPLOAD_AD
    )

    # Add the ad group ad to the client account and display the resulting
    # ad's resource name.
    mutate_ad_group_ads_response: MutateAdGroupAdsResponse = (
        ad_group_ad_service.mutate_ad_group_ads(
            customer_id=customer_id, operations=[ad_group_ad_operation]
        )
    )
    print(
        "Created new ad group ad with resource name "
        f"'{mutate_ad_group_ads_response.results[0].resource_name}'."
    )


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Adds a display upload ad to a given ad group."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-a",
        "--ad_group_id",
        type=str,
        required=True,
        help="The ID of the ad group to which the new ad will be added.",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.customer_id, args.ad_group_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.errors",
    marshal="google.ads.googleads.v19",
    manifest={
        "CampaignFeedErrorEnum",
    },
)


class CampaignFeedErrorEnum(proto.Message):
    r"""Container for enum describing possible campaign feed errors."""

    class CampaignFeedError(proto.Enum):
        r"""Enum describing possible campaign feed errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE (2):
                An active feed already exists for this
                campaign and placeholder type.
            CANNOT_CREATE_FOR_REMOVED_FEED (4):
                The specified feed is removed.
            CANNOT_CREATE_ALREADY_EXISTING_CAMPAIGN_FEED (5):
                The CampaignFeed already exists. UPDATE
                should be used to modify the existing
                CampaignFeed.
            CANNOT_MODIFY_REMOVED_CAMPAIGN_FEED (6):
                Cannot update removed campaign feed.
            INVALID_PLACEHOLDER_TYPE (7):
                Invalid placeholder type.
            MISSING_FEEDMAPPING_FOR_PLACEHOLDER_TYPE (8):
                Feed mapping for this placeholder type does
                not exist.
            NO_EXISTING_LOCATION_CUSTOMER_FEED (9):
                Location CampaignFeeds cannot be created
                unless there is a location CustomerFeed for the
                specified feed.
            LEGACY_FEED_TYPE_READ_ONLY (10):
                Feed is read only.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE = 2
        CANNOT_CREATE_FOR_REMOVED_FEED = 4
        CANNOT_CREATE_ALREADY_EXISTING_CAMPAIGN_FEED = 5
        CANNOT_MODIFY_REMOVED_CAMPAIGN_FEED = 6
        INVALID_PLACEHOLDER_TYPE = 7
        MISSING_FEEDMAPPING_FOR_PLACEHOLDER_TYPE = 8
        NO_EXISTING_LOCATION_CUSTOMER_FEED = 9
        LEGACY_FEED_TYPE_READ_ONLY = 10


__all__ = tuple(sorted(__protobuf__.manifest))
